@@ -1,9 +1,9 @@
 /*****************************************************************************
- * charset.h: Unicode UTF-8 wrappers function
+ * vlc_charset.h: Unicode UTF-8 wrappers function
  *****************************************************************************
  * Copyright (C) 2003-2005 VLC authors and VideoLAN
  * Copyright © 2005-2010 Rémi Denis-Courmont
- * $Id: cd4d40725344eda07f51dd16f11aeadc351141e8 $
+ * $Id: 3119e89fd6bad58096a18211009d1504b97c9fbe $
  *
  * Author: Rémi Denis-Courmont <rem # videolan,org>
  *
@@ -38,12 +38,6 @@ VLC_API int vlc_iconv_close( vlc_iconv_t );
 
 #include <stdarg.h>
 
-VLC_API void LocaleFree( const char * );
-VLC_API char * FromLocale( const char * ) VLC_USED;
-VLC_API char * FromLocaleDup( const char * ) VLC_USED;
-VLC_API char * ToLocale( const char * ) VLC_USED;
-VLC_API char * ToLocaleDup( const char * ) VLC_USED;
-
 VLC_API int utf8_vfprintf( FILE *stream, const char *fmt, va_list ap );
 VLC_API int utf8_fprintf( FILE *, const char *, ... ) VLC_FORMAT( 2, 3 );
 VLC_API char * vlc_strcasestr(const char *, const char *) VLC_USED;
@@ -51,7 +45,10 @@ VLC_API char * vlc_strcasestr(const char *, const char *) VLC_USED;
 VLC_API char * EnsureUTF8( char * );
 VLC_API const char * IsUTF8( const char * ) VLC_USED;
 
-#ifdef WIN32
+VLC_API char * FromCharset( const char *charset, const void *data, size_t data_size ) VLC_USED;
+VLC_API void * ToCharset( const char *charset, const char *in, size_t *outsize ) VLC_USED;
+
+#ifdef _WIN32
 VLC_USED
 static inline char *FromWide (const wchar_t *wide)
 {
@@ -79,6 +76,105 @@ static inline wchar_t *ToWide (const char *utf8)
         MultiByteToWideChar (CP_UTF8, 0, utf8, -1, out, len);
     return out;
 }
+
+VLC_USED VLC_MALLOC
+static inline char *ToCodePage (unsigned cp, const char *utf8)
+{
+    wchar_t *wide = ToWide (utf8);
+    if (wide == NULL)
+        return NULL;
+
+    size_t len = WideCharToMultiByte (cp, 0, wide, -1, NULL, 0, NULL, NULL);
+    if (len == 0) {
+        free(wide);
+        return NULL;
+    }
+
+    char *out = (char *)malloc (len);
+    if (likely(out != NULL))
+        WideCharToMultiByte (cp, 0, wide, -1, out, len, NULL, NULL);
+    free (wide);
+    return out;
+}
+
+VLC_USED VLC_MALLOC
+static inline char *FromCodePage (unsigned cp, const char *mb)
+{
+    int len = MultiByteToWideChar (cp, 0, mb, -1, NULL, 0);
+    if (len == 0)
+        return NULL;
+
+    wchar_t *wide = (wchar_t *)malloc (len * sizeof (wchar_t));
+    if (unlikely(wide == NULL))
+        return NULL;
+    MultiByteToWideChar (cp, 0, mb, -1, wide, len);
+
+    char *utf8 = FromWide (wide);
+    free (wide);
+    return utf8;
+}
+
+VLC_USED VLC_MALLOC
+static inline char *FromANSI (const char *ansi)
+{
+    return FromCodePage (GetACP (), ansi);
+}
+
+VLC_USED VLC_MALLOC
+static inline char *ToANSI (const char *utf8)
+{
+    return ToCodePage (GetACP (), utf8);
+}
+
+# ifdef UNICODE
+#  define FromT FromWide
+#  define ToT   ToWide
+# else
+#  define FromT FromANSI
+#  define ToT   ToANSI
+# endif
+# define FromLocale    FromANSI
+# define ToLocale      ToANSI
+# define LocaleFree(s) free((char *)(s))
+# define FromLocaleDup FromANSI
+# define ToLocaleDup   ToANSI
+
+#elif defined(__OS2__)
+
+VLC_USED static inline char *FromLocale (const char *locale)
+{
+    return locale ? FromCharset ((char *)"", locale, strlen(locale)) : NULL;
+}
+
+VLC_USED static inline char *ToLocale (const char *utf8)
+{
+    size_t outsize;
+    return utf8 ? (char *)ToCharset ("", utf8, &outsize) : NULL;
+}
+
+VLC_USED static inline void LocaleFree (const char *str)
+{
+    free ((char *)str);
+}
+
+VLC_USED static inline char *FromLocaleDup (const char *locale)
+{
+    return FromCharset ("", locale, strlen(locale));
+}
+
+VLC_USED static inline char *ToLocaleDup (const char *utf8)
+{
+    size_t outsize;
+    return (char *)ToCharset ("", utf8, &outsize);
+}
+
+#else
+
+# define FromLocale(l) (l)
+# define ToLocale(u)   (u)
+# define LocaleFree(s) ((void)(s))
+# define FromLocaleDup strdup
+# define ToLocaleDup   strdup
 #endif
 
 /**
@@ -108,9 +204,6 @@ static inline char *FromLatin1 (const char *latin)
     return utf8 ? utf8 : str;
 }
 
-VLC_API char * FromCharset( const char *charset, const void *data, size_t data_size ) VLC_USED;
-VLC_API void * ToCharset( const char *charset, const char *in, size_t *outsize ) VLC_USED;
-
 VLC_API double us_strtod( const char *, char ** ) VLC_USED;
 VLC_API float us_strtof( const char *, char ** ) VLC_USED;
 VLC_API double us_atof( const char * ) VLC_USED;
