@@ -2,7 +2,7 @@
  * playlist.c: libvlc new API playlist handling functions
  *****************************************************************************
  * Copyright (C) 2005 VLC authors and VideoLAN
- * $Id: 698d66b49066e00f76f781a3a6ce8cbaf6d10b94 $
+ * $Id: c0e610fa528a709e6a1e66559272784e698dd64f $
  *
  * Authors: Clément Stenac <zorglub@videolan.org>
  *
@@ -41,20 +41,12 @@
 void libvlc_playlist_play( libvlc_instance_t *p_instance, int i_id,
                            int i_options, char **ppsz_options )
 {
-    playlist_t *pl = pl_Get (p_instance->p_libvlc_int);
+    libvlc_InternalPlay( p_instance->p_libvlc_int );
     VLC_UNUSED(i_id); VLC_UNUSED(i_options); VLC_UNUSED(ppsz_options);
-
-    assert( pl );
-    if( !var_GetBool( pl, "playlist-autostart" )
-     || pl->items.i_size == 0 )
-        return;
-    playlist_Control( pl, PLAYLIST_PLAY, false );
 }
 
 int libvlc_add_intf( libvlc_instance_t *p_instance, const char *name )
 {
-    pl_Get (p_instance->p_libvlc_int);
-
     if( libvlc_InternalAddIntf( p_instance->p_libvlc_int, name ))
     {
         if( name != NULL )
