@@ -26,7 +26,6 @@
 #endif
 #include <stdarg.h>
 #include <assert.h>
-#include <errno.h>
 
 #include <vlc_common.h>
 #include <vlc_demux.h>
@@ -290,12 +289,13 @@ static int Open (vlc_object_t *obj)
         }
 
         char *salt = var_CreateGetNonEmptyString (demux, "srtp-salt");
-        errno = srtp_setkeystring (p_sys->srtp, key, salt ? salt : "");
+        int val = srtp_setkeystring (p_sys->srtp, key, salt ? salt : "");
         free (salt);
         free (key);
-        if (errno)
+        if (val)
         {
-            msg_Err (obj, "bad SRTP key/salt combination (%m)");
+            msg_Err (obj, "bad SRTP key/salt combination (%s)",
+                     vlc_strerror_c(val));
             goto error;
         }
     }
