@@ -2,7 +2,7 @@
  * zipstream.c: stream_filter that creates a XSPF playlist from a Zip archive
  *****************************************************************************
  * Copyright (C) 2009 VLC authors and VideoLAN
- * $Id: 3bb9232caa6374d72585e155f6b99ef162c7170c $
+ * $Id: 8820a799198be195b87c5d083d9df9c8f756cd21 $
  *
  * Authors: Jean-Philippe André <jpeg@videolan.org>
  *
@@ -35,9 +35,6 @@
 /* FIXME remove */
 #include <vlc_input.h>
 
-#define FILENAME_TEXT N_( "Media in Zip" )
-#define FILENAME_LONGTEXT N_( "Path to the media in the Zip archive" )
-
 /** **************************************************************************
  * Module descriptor
  *****************************************************************************/
@@ -322,7 +319,6 @@ static int Control( stream_t *s, int i_query, va_list args )
             return VLC_EGENERIC;
 
         case STREAM_UPDATE_SIZE:
-        case STREAM_CONTROL_ACCESS:
         case STREAM_CAN_SEEK:
         case STREAM_CAN_FASTSEEK:
         case STREAM_SET_RECORD_STATE:
@@ -440,6 +436,8 @@ static int GetFilesInZip( stream_t *p_this, unzFile file,
             != UNZ_OK )
         {
             msg_Warn( p_this, "can't get info about file in zip" );
+            free( psz_fileName );
+            free( p_fileInfo );
             return VLC_EGENERIC;
         }
 
@@ -718,7 +716,9 @@ static node* findOrCreateParentNode( node *root, const char *fullpath )
         if( current->name && !strcmp( current->name, folder ) )
         {
             /* We found the folder, go recursively deeper */
-            return findOrCreateParentNode( current, sep );
+            node *parentNode = findOrCreateParentNode( current, sep );
+            free( path );
+            return parentNode;
         }
         current = current->next;
     }
