@@ -2,7 +2,7 @@
  * adpcm.c : adpcm variant audio decoder
  *****************************************************************************
  * Copyright (C) 2001, 2002 VLC authors and VideoLAN
- * $Id: cdeb5b067eb90419d33ddca01b6ae478a30dd1a6 $
+ * $Id: e655c45ca990e3171f3d6f356754473b19d6af30 $
  *
  * Authors: Laurent Aimar <fenrir@via.ecp.fr>
  *          Rémi Denis-Courmont <rem # videolan.org>
@@ -72,6 +72,7 @@ struct decoder_sys_t
     size_t              i_samplesperblock;
 
     date_t              end_date;
+    int16_t            *prev;
 };
 
 static void DecodeAdpcmMs    ( decoder_t *, int16_t *, uint8_t * );
@@ -141,8 +142,8 @@ static int OpenDecoder( vlc_object_t *p_this )
         case VLC_FOURCC('i','m','a', '4'): /* IMA ADPCM */
         case VLC_FOURCC('m','s',0x00,0x02): /* MS ADPCM */
         case VLC_FOURCC('m','s',0x00,0x11): /* IMA ADPCM */
-        case VLC_FOURCC('m','s',0x00,0x61): /* Duck DK4 ADPCM */
-        case VLC_FOURCC('m','s',0x00,0x62): /* Duck DK3 ADPCM */
+        case VLC_CODEC_ADPCM_DK3:
+        case VLC_CODEC_ADPCM_DK4:
         case VLC_FOURCC('X','A','J', 0): /* EA ADPCM */
             break;
         default:
@@ -164,10 +165,12 @@ static int OpenDecoder( vlc_object_t *p_this )
     }
 
     /* Allocate the memory needed to store the decoder's structure */
-    if( ( p_dec->p_sys = p_sys =
-          (decoder_sys_t *)malloc(sizeof(decoder_sys_t)) ) == NULL )
+    p_sys = malloc(sizeof(*p_sys));
+    if( unlikely(p_sys == NULL) )
         return VLC_ENOMEM;
 
+    p_sys->prev = NULL;
+
     switch( p_dec->fmt_in.i_codec )
     {
         case VLC_FOURCC('i','m','a', '4'): /* IMA ADPCM */
@@ -179,17 +182,17 @@ static int OpenDecoder( vlc_object_t *p_this )
         case VLC_CODEC_ADPCM_MS: /* MS ADPCM */
             p_sys->codec = ADPCM_MS;
             break;
-        case VLC_FOURCC('m','s',0x00,0x61): /* Duck DK4 ADPCM */
+        case VLC_CODEC_ADPCM_DK4: /* Duck DK4 ADPCM */
             p_sys->codec = ADPCM_DK4;
             break;
-        case VLC_FOURCC('m','s',0x00,0x62): /* Duck DK3 ADPCM */
+        case VLC_CODEC_ADPCM_DK3: /* Duck DK3 ADPCM */
             p_sys->codec = ADPCM_DK3;
             break;
         case VLC_FOURCC('X','A','J', 0): /* EA ADPCM */
             p_sys->codec = ADPCM_EA;
-            p_dec->fmt_in.p_extra = calloc( 2 * p_dec->fmt_in.audio.i_channels,
-                                            sizeof( int16_t ) );
-            if( p_dec->fmt_in.p_extra == NULL )
+            p_sys->prev = calloc( 2 * p_dec->fmt_in.audio.i_channels,
+                                  sizeof( int16_t ) );
+            if( unlikely(p_sys->prev == NULL) )
             {
                 free( p_sys );
                 return VLC_ENOMEM;
@@ -245,6 +248,7 @@ static int OpenDecoder( vlc_object_t *p_this )
              p_dec->fmt_in.audio.i_bitspersample, p_sys->i_block,
              p_sys->i_samplesperblock );
 
+    p_dec->p_sys = p_sys;
     p_dec->fmt_out.i_cat = AUDIO_ES;
     p_dec->fmt_out.i_codec = VLC_CODEC_S16N;
     p_dec->fmt_out.audio.i_rate = p_dec->fmt_in.audio.i_rate;
@@ -349,8 +353,7 @@ static void CloseDecoder( vlc_object_t *p_this )
     decoder_t *p_dec = (decoder_t *)p_this;
     decoder_sys_t *p_sys = p_dec->p_sys;
 
-    if( p_sys->codec == ADPCM_EA )
-        free( p_dec->fmt_in.p_extra );
+    free( p_sys->prev );
     free( p_sys );
 }
 
@@ -729,65 +732,56 @@ static void DecodeAdpcmDk3( decoder_t *p_dec, int16_t *p_sample,
 static void DecodeAdpcmEA( decoder_t *p_dec, int16_t *p_sample,
                            uint8_t *p_buffer )
 {
-    static const uint32_t EATable[]=
+    static const int16_t EATable[]=
     {
-        0x00000000, 0x000000F0, 0x000001CC, 0x00000188,
-        0x00000000, 0x00000000, 0xFFFFFF30, 0xFFFFFF24,
-        0x00000000, 0x00000001, 0x00000003, 0x00000004,
-        0x00000007, 0x00000008, 0x0000000A, 0x0000000B,
-        0x00000000, 0xFFFFFFFF, 0xFFFFFFFD, 0xFFFFFFFC
+        0x0000, 0x00F0, 0x01CC, 0x0188, 0x0000, 0x0000, 0xFF30, 0xFF24,
+        0x0000, 0x0001, 0x0003, 0x0004, 0x0007, 0x0008, 0x000A, 0x000B,
+        0x0000, 0xFFFF, 0xFFFD, 0xFFFC,
     };
     decoder_sys_t *p_sys  = p_dec->p_sys;
-    uint8_t *p_end;
-    unsigned i_channels, c;
-    int16_t *prev, *cur;
-    int32_t c1[MAX_CHAN], c2[MAX_CHAN];
-    int8_t d[MAX_CHAN];
+    int_fast32_t c1[MAX_CHAN], c2[MAX_CHAN];
+    int_fast8_t d[MAX_CHAN];
 
-    i_channels = p_dec->fmt_in.audio.i_channels;
-    p_end = &p_buffer[p_sys->i_block];
+    unsigned chans = p_dec->fmt_in.audio.i_channels;
+    const uint8_t *p_end = &p_buffer[p_sys->i_block];
+    int16_t *prev = p_sys->prev;
+    int16_t *cur = prev + chans;
 
-    prev = (int16_t *)p_dec->fmt_in.p_extra;
-    cur = prev + i_channels;
-
-    for (c = 0; c < i_channels; c++)
+    for (unsigned c = 0; c < chans; c++)
     {
-        uint8_t input;
+        uint8_t input = p_buffer[c];
 
-        input = p_buffer[c];
         c1[c] = EATable[input >> 4];
         c2[c] = EATable[(input >> 4) + 4];
         d[c] = (input & 0xf) + 8;
     }
 
-    for( p_buffer += i_channels; p_buffer < p_end ; p_buffer += i_channels)
+    for (p_buffer += chans; p_buffer < p_end; p_buffer += chans)
     {
-        for (c = 0; c < i_channels; c++)
+        union { uint32_t u; int32_t i; } spl;
+
+        for (unsigned c = 0; c < chans; c++)
         {
-            int32_t spl;
-
-            spl = (p_buffer[c] >> 4) & 0xf;
-            spl = (spl << 0x1c) >> d[c];
-            spl = (spl + cur[c] * c1[c] + prev[c] * c2[c] + 0x80) >> 8;
-            CLAMP( spl, -32768, 32767 );
+            spl.u = (p_buffer[c] & 0xf0u) << 24u;
+            spl.i >>= d[c];
+            spl.i = (spl.i + cur[c] * c1[c] + prev[c] * c2[c] + 0x80) >> 8;
+            CLAMP(spl.i, -32768, 32767);
             prev[c] = cur[c];
-            cur[c] = spl;
+            cur[c] = spl.i;
 
-            *(p_sample++) = spl;
+            *(p_sample++) = spl.i;
         }
 
-        for (c = 0; c < i_channels; c++)
+        for (unsigned c = 0; c < chans; c++)
         {
-            int32_t spl;
-
-            spl = p_buffer[c] & 0xf;
-            spl = (spl << 0x1c) >> d[c];
-            spl = (spl + cur[c] * c1[c] + prev[c] * c2[c] + 0x80) >> 8;
-            CLAMP( spl, -32768, 32767 );
+            spl.u = (p_buffer[c] & 0x0fu) << 28u;
+            spl.i >>= d[c];
+            spl.i = (spl.i + cur[c] * c1[c] + prev[c] * c2[c] + 0x80) >> 8;
+            CLAMP(spl.i, -32768, 32767);
             prev[c] = cur[c];
-            cur[c] = spl;
+            cur[c] = spl.i;
 
-            *(p_sample++) = spl;
+            *(p_sample++) = spl.i;
         }
     }
 }
