@@ -2,7 +2,7 @@
  * video.c: video decoder using the libavcodec library
  *****************************************************************************
  * Copyright (C) 1999-2001 VLC authors and VideoLAN
- * $Id: a063754229315e3383bca4dce2aa6c026aa26bc0 $
+ * $Id: 0ecd7fec5484e8386652e5458fa3b888ce7d08e8 $
  *
  * Authors: Laurent Aimar <fenrir@via.ecp.fr>
  *          Gildas Bazin <gbazin@videolan.org>
@@ -191,6 +191,17 @@ static inline picture_t *ffmpeg_NewPictBuf( decoder_t *p_dec,
         p_dec->fmt_out.video.i_frame_rate_base =
             p_dec->fmt_in.video.i_frame_rate_base;
     }
+#if LIBAVCODEC_VERSION_CHECK( 56, 5, 0, 7, 100 )
+    else if( p_context->framerate.num > 0 && p_context->framerate.den > 0 )
+    {
+        p_dec->fmt_out.video.i_frame_rate = p_context->framerate.num;
+        p_dec->fmt_out.video.i_frame_rate_base = p_context->framerate.den;
+# if LIBAVCODEC_VERSION_MICRO <  100
+        // for some reason libav don't thinkg framerate presents actually same thing as in ffmpeg
+        p_dec->fmt_out.video.i_frame_rate_base *= __MAX( p_context->ticks_per_frame, 1 );
+# endif
+    }
+#endif
     else if( p_context->time_base.num > 0 && p_context->time_base.den > 0 )
     {
         p_dec->fmt_out.video.i_frame_rate = p_context->time_base.den;
