@@ -2,7 +2,7 @@
  * scte27.c : SCTE-27 subtitles decoder
  *****************************************************************************
  * Copyright (C) Laurent Aimar
- * $Id: a348da5a9268dd416c13156ddd67360110ee4411 $
+ * $Id: 527e4e45048217fcb2e2157928c02036ef37d00a $
  *
  * Authors: Laurent Aimar <fenrir _AT_ videolan _DOT_ org>
  *
@@ -252,7 +252,9 @@ static subpicture_region_t *DecodeSimpleBitmap(decoder_t *dec,
     video_format_t fmt = {
         .i_chroma = VLC_CODEC_YUVP,
         .i_width = frame_h,
+        .i_visible_width = frame_h,
         .i_height = frame_v,
+        .i_visible_height = frame_v,
         .i_sar_num = 0, /* Use video AR */
         .i_sar_den = 1,
         .p_palette = &palette,
