@@ -2,7 +2,7 @@
  * es.c : Generic audio ES input module for vlc
  *****************************************************************************
  * Copyright (C) 2001-2008 VLC authors and VideoLAN
- * $Id: 8b4b7d2bcf469e3275feb105be3afcfedcfb463f $
+ * $Id: 819b89be9b54feb03ccee5c70dc72aa9f286699b $
  *
  * Authors: Laurent Aimar <fenrir@via.ecp.fr>
  *          Gildas Bazin <gbazin@videolan.org>
@@ -332,28 +332,28 @@ static int Control( demux_t *p_demux, int i_query, va_list args )
     int64_t *pi64;
     bool *pb_bool;
     int i_ret;
-    va_list args_save;
-
-    va_copy ( args_save, args );
 
     switch( i_query )
     {
         case DEMUX_HAS_UNSUPPORTED_META:
             pb_bool = (bool*)va_arg( args, bool* );
             *pb_bool = true;
-            va_end( args_save );
             return VLC_SUCCESS;
 
         case DEMUX_GET_TIME:
             pi64 = (int64_t*)va_arg( args, int64_t * );
             *pi64 = p_sys->i_pts + p_sys->i_time_offset;
-            va_end( args_save );
             return VLC_SUCCESS;
 
         case DEMUX_GET_LENGTH:
-            i_ret = demux_vaControlHelper( p_demux->s, p_sys->i_stream_offset, -1,
-                                            p_sys->i_bitrate_avg, 1, i_query,
-                                            args );
+        {
+            va_list ap;
+
+            va_copy ( ap, args );
+            i_ret = demux_vaControlHelper( p_demux->s, p_sys->i_stream_offset,
+                                    -1, p_sys->i_bitrate_avg, 1, i_query, ap );
+            va_end( ap );
+
             /* No bitrate, we can't have it precisely, but we can compute
              * a raw approximation with time/position */
             if( i_ret && !p_sys->i_bitrate_avg )
@@ -365,17 +365,15 @@ static int Control( demux_t *p_demux, int i_query, va_list args )
                 if( f_pos < 0.01 ||
                     (p_sys->i_pts + p_sys->i_time_offset) < 8000000 )
                 {
-                    va_end( args_save );
                     return VLC_EGENERIC;
                 }
 
-                pi64 = (int64_t *)va_arg( args_save, int64_t * );
+                pi64 = (int64_t *)va_arg( args, int64_t * );
                 *pi64 = (p_sys->i_pts + p_sys->i_time_offset) / f_pos;
-                va_end( args_save );
                 return VLC_SUCCESS;
             }
-            va_end( args_save );
             return i_ret;
+        }
 
         case DEMUX_SET_TIME:
             /* FIXME TODO: implement a high precision seek (with mp3 parsing)
@@ -384,7 +382,6 @@ static int Control( demux_t *p_demux, int i_query, va_list args )
             i_ret = demux_vaControlHelper( p_demux->s, p_sys->i_stream_offset, -1,
                                             p_sys->i_bitrate_avg, 1, i_query,
                                             args );
-            va_end( args_save );
             if( !i_ret && p_sys->i_bitrate_avg > 0 &&
                 (i_query == DEMUX_SET_POSITION || i_query == DEMUX_SET_TIME) )
             {
@@ -615,7 +612,12 @@ static int GenericProbe( demux_t *p_demux, int64_t *pi_offset,
             for( int t = 0; t < 1 + !!b_wav; t++ )
             {
                 if( t == 1 )
+                {
+                    if(!i_samples)
+                        break;
                     i_size = i_samples * 2 * 2;
+                }
+
                 if( i_skip + i_check_size + i_size <= i_peek )
                 {
                     b_ok = pf_check( &p_peek[i_skip+i_size], NULL ) >= 0;
