@@ -2,7 +2,7 @@
  * m3u.c : M3U playlist format import
  *****************************************************************************
  * Copyright (C) 2004 VLC authors and VideoLAN
- * $Id: c2e1e0f4d2fb05281835582fd5dababe62e41d9a $
+ * $Id: 58f6006deb41b13fe9fac8b90cbdb8d8961bb1bf $
  *
  * Authors: Clément Stenac <zorglub@videolan.org>
  *          Sigmund Augdal Helberg <dnumgis@videolan.org>
@@ -129,7 +129,9 @@ static bool ContainsURL( demux_t *p_demux )
             !strncasecmp( (const char *)p_peek, "mms://", 6 ) ||
             !strncasecmp( (const char *)p_peek, "rtsp://", 7 ) ||
             !strncasecmp( (const char *)p_peek, "https://", 8 ) ||
-            !strncasecmp( (const char *)p_peek, "ftp://", 6 ) )
+            !strncasecmp( (const char *)p_peek, "ftp://", 6 ) ||
+            !strncasecmp( (const char *)p_peek, "ftps://", 7 ) ||
+            !strncasecmp( (const char *)p_peek, "ftpes://", 8 ) )
         {
             return true;
         }
