@@ -158,6 +158,11 @@ static CGEventRef tapEventCallback(CGEventTapProxy proxy, CGEventType type, CGEv
              @"com.jriver.MediaCenter18",
              @"com.jriver.MediaCenter19",
              @"com.jriver.MediaCenter20",
+             @"co.rackit.mate",
+             @"com.ttitt.b-music",
+             @"com.beardedspice.BeardedSpice",
+             @"com.plug.Plug",
+             @"com.netease.163music",
             nil
     ];
 }
@@ -251,7 +256,6 @@ static CGEventRef tapEventCallback(CGEventTapProxy proxy, CGEventType type, CGEv
     [_delegate mediaKeyTap:self receivedMediaKeyEvent:event];
 }
 
-
 -(void)eventTapThread;
 {
     _tapThreadRL = CFRunLoopGetCurrent();
