@@ -1,8 +1,8 @@
 /*****************************************************************************
  * coredialogs.h: Mac OS X Core Dialogs
  *****************************************************************************
- * Copyright (C) 2005-2011 VLC authors and VideoLAN
- * $Id: 4a73039dd7a7ef9ad4f1f8ecf62cd622a490077c $
+ * Copyright (C) 2005-2012 VLC authors and VideoLAN
+ * $Id: 2c9624713d0edfb27652b1328180a9b339214a74 $
  *
  * Authors: Derk-Jan Hartman <hartman at videolan dot org>
  *          Felix Paul Kühne <fkuehne at videolan dot org>
@@ -35,7 +35,6 @@
     IBOutlet id o_window;
     IBOutlet id o_cleanup_button;
     IBOutlet id o_error_table;
-    IBOutlet id o_messages_btn;
 
     NSMutableArray * o_errors;
     NSMutableArray * o_icons;
@@ -43,7 +42,6 @@
     BOOL b_nib_loaded;
 }
 - (IBAction)cleanupTable:(id)sender;
-- (IBAction)showMessages:(id)sender;
 
 -(void)showPanel;
 -(void)addError: (NSString *)o_error withMsg:(NSString *)o_msg;
@@ -78,6 +76,8 @@
 }
 + (VLCCoreDialogProvider *)sharedInstance;
 
+@property (atomic,readwrite) BOOL progressCancelled;
+
 -(void)performEventWithObject: (NSValue *)o_value ofType: (const char*)type;
 
 -(void)showFatalDialog: (NSValue *)o_value;
@@ -90,7 +90,6 @@
 -(void)showProgressDialogOnMainThread: (NSValue *)o_value;
 -(void)showProgressDialog: (NSValue *)o_value;
 -(IBAction)progDialogAction:(id)sender;
--(BOOL)progressCancelled;
 -(void)updateProgressPanelWithText: (NSString *)string andNumber: (double)d_number;
 -(void)destroyProgressPanel;
 
