@@ -1,11 +1,12 @@
 /*****************************************************************************
  * fspanel.h: MacOS X full screen panel
  *****************************************************************************
- * Copyright (C) 2006-2011 VLC authors and VideoLAN
- * $Id: a3d7dd462479b29558261a3198a05eb50d8270ec $
+ * Copyright (C) 2006-2013 VLC authors and VideoLAN
+ * $Id: 614dfff8b24b1e1e5962b72600c0e2daca55af98 $
  *
  * Authors: Jérôme Decoodt <djc at videolan dot org>
  *          Felix Paul Kühne <fkuehne at videolan dot org>
+ *          David Fuhrmann <david dot fuhrmann at googlemail dot com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -22,6 +23,8 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
  *****************************************************************************/
 
+@class VLCWindow;
+
 @interface VLCFSPanel : NSWindow
 {
     NSTimer *fadeTimer,*hideAgainTimer;
@@ -35,19 +38,20 @@
     BOOL b_displayed;
     BOOL b_voutWasUpdated;
     int i_device;
+
+    VLCWindow *o_vout_window;
 }
 - (id)initWithContentRect: (NSRect)contentRect
                 styleMask: (NSUInteger)aStyle
                   backing: (NSBackingStoreType)bufferingType
                     defer: (BOOL)flag;
-- (void)awakeFromNib;
 - (BOOL)canBecomeKeyWindow;
 - (void)dealloc;
 
 - (void)setPlay;
 - (void)setPause;
 - (void)setStreamTitle: (NSString *)o_title;
-- (void)setStreamPos: (float)f_pos andTime: (NSString *)o_time;
+- (void)updatePositionAndTime;
 - (void)setSeekable: (BOOL)b_seekable;
 - (void)setVolumeLevel: (int)i_volumeLevel;
 
@@ -69,16 +73,21 @@
 - (void)mouseDown: (NSEvent *)theEvent;
 - (void)mouseDragged: (NSEvent *)theEvent;
 
-- (BOOL)isDisplayed;
-- (void)setVoutWasUpdated: (int)i_screen;
+- (void)setVoutWasUpdated: (VLCWindow *)o_window;
 @end
 
+@class VLCProgressView;
+@class VLCFSVolumeSlider;
+
 @interface VLCFSPanelView : NSView
 {
     NSColor *fillColor;
     NSButton *o_prev, *o_next, *o_bwd, *o_fwd, *o_play, *o_fullscreen;
-    NSTextField *o_streamTitle_txt, *o_streamPosition_txt;
-    NSSlider *o_fs_timeSlider, *o_fs_volumeSlider;
+    NSTextField *o_streamTitle_txt;
+    VLCTimeField *o_streamPosition_txt, *o_streamLength_txt;
+    NSSlider *o_fs_timeSlider;
+    VLCFSVolumeSlider *o_fs_volumeSlider;
+    VLCProgressView *o_progress_view;
     NSImage *o_background_img, *o_vol_sld_img, *o_vol_mute_img, *o_vol_max_img, *o_time_sld_img;
     NSTimeInterval last_fwd_event;
     NSTimeInterval last_bwd_event;
@@ -89,7 +98,7 @@
 - (void)setPlay;
 - (void)setPause;
 - (void)setStreamTitle: (NSString *)o_title;
-- (void)setStreamPos: (float)f_pos andTime: (NSString *)o_time;
+- (void)updatePositionAndTime;
 - (void)setSeekable: (BOOL)b_seekable;
 - (void)setVolumeLevel: (int)i_volumeLevel;
 - (IBAction)play:(id)sender;
@@ -110,7 +119,7 @@
 
 @end
 
-@interface VLCFSVolumeSlider : NSSlider
+@interface VLCFSVolumeSlider : VLCVolumeSliderCommon
 {
 }
 - (void)drawKnobInRect: (NSRect)knobRect;
