@@ -1,8 +1,8 @@
 /*****************************************************************************
- * ToolbarEdit.hpp : ToolbarEdit dialogs
+ * toolbar.hpp : ToolbarEdit dialogs
  ****************************************************************************
  * Copyright (C) 2008-2011 the VideoLAN team
- * $Id: 0b8dc44de887d43e53c78d9daab13fd666225f0b $
+ * $Id: 276decb2001857542e1321466b6cf5967841d29f $
  *
  * Authors: Jean-Baptiste Kempf <jb (at) videolan.org>
  *
@@ -48,6 +48,25 @@ class DroppingController;
 class QComboBox;
 class QRubberBand;
 
+class PreviewWidget : public QWidget
+{
+    Q_OBJECT
+
+public:
+    PreviewWidget( QWidget *, QWidget *, QWidget * );
+
+public slots:
+    void setBarsTopPosition( int b );
+
+protected:
+    virtual void paintEvent(QPaintEvent *);
+    virtual bool eventFilter(QObject *obj, QEvent *event);
+
+private:
+    QWidget * bars[3];
+    bool b_top;
+};
+
 class WidgetListing : public QListWidget
 {
     Q_OBJECT
@@ -70,8 +89,8 @@ public:
     virtual ~ToolbarEditDialog();
 
 private:
-    QCheckBox *flatBox, *bigBox, *shinyBox;
-    QComboBox *positionCombo, *profileCombo;
+    QCheckBox *flatBox, *bigBox, *shinyBox, *positionCheckbox;
+    QComboBox *profileCombo;
 
     WidgetListing *widgetListing;
     DroppingController *controller1, *controller2, *controllerA;
