@@ -2,7 +2,7 @@
  * dialogs_provider.cpp : Dialog Provider
  *****************************************************************************
  * Copyright (C) 2006-2009 the VideoLAN team
- * $Id: dbdcef03cc67567e7c7436b019ce7f044f695aed $
+ * $Id: 23c336dc5d321402362ac0af4d364f5234415e1a $
  *
  * Authors: Clément Stenac <zorglub@videolan.org>
  *          Jean-Baptiste Kempf <jb@videolan.org>
@@ -440,16 +440,14 @@ QStringList DialogsProvider::showSimpleOpen( const QString& help,
 void DialogsProvider::addFromSimple( bool pl, bool go)
 {
     QStringList files = DialogsProvider::showSimpleOpen();
-    int mode = go ? PLAYLIST_GO : PLAYLIST_PREPARSE;
 
+    bool first = go;
     files.sort();
     foreach( const QString &file, files )
     {
         QString url = toURI( toNativeSeparators( file ) );
-        playlist_Add( THEPL, qtu( url ), NULL, PLAYLIST_APPEND | mode,
-                      PLAYLIST_END, pl, pl_Unlocked );
-        RecentsMRL::getInstance( p_intf )->addRecent( url );
-        mode = PLAYLIST_PREPARSE;
+        Open::openMRL( p_intf, url, first, pl);
+        first = false;
     }
 }
 
@@ -458,16 +456,6 @@ void DialogsProvider::simpleOpenDialog()
     addFromSimple( true, true ); /* Playlist and Go */
 }
 
-void DialogsProvider::simplePLAppendDialog()
-{
-    addFromSimple( true, false );
-}
-
-void DialogsProvider::simpleMLAppendDialog()
-{
-    addFromSimple( false, false );
-}
-
 /* Url & Clipboard */
 /**
  * Open a MRL.
@@ -491,11 +479,8 @@ void DialogsProvider::openUrlDialog()
         url = qfu(uri);
         free( uri );
     }
-    playlist_Add( THEPL, qtu(url), NULL,
-                  !oud.shouldEnqueue() ? ( PLAYLIST_APPEND | PLAYLIST_GO )
-                                     : ( PLAYLIST_APPEND | PLAYLIST_PREPARSE ),
-                  PLAYLIST_END, true, false );
-    RecentsMRL::getInstance( p_intf )->addRecent( url );
+
+    Open::openMRL( p_intf, qtu(url), !oud.shouldEnqueue() );
 }
 
 /* Directory */
@@ -506,17 +491,24 @@ void DialogsProvider::openUrlDialog()
  **/
 static void openDirectory( intf_thread_t *p_intf, bool pl, bool go )
 {
-    QString dir = QFileDialog::getExistingDirectory( NULL, qtr( I_OP_DIR_WINTITLE ), p_intf->p_sys->filepath );
+    QString uri = DialogsProvider::getDirectoryDialog( p_intf );
+    if( !uri.isEmpty() )
+        Open::openMRL( p_intf, uri, go, pl );
+}
 
-    if( dir.isEmpty() )
-        return;
+QString DialogsProvider::getDirectoryDialog( intf_thread_t *p_intf )
+{
+    QString dir = QFileDialog::getExistingDirectory( NULL,
+            qtr( I_OP_DIR_WINTITLE ), p_intf->p_sys->filepath );
+
+    if( dir.isEmpty() ) return QString();
 
     p_intf->p_sys->filepath = dir;
 
     const char *scheme = "directory";
-    if( dir.endsWith( "VIDEO_TS", Qt::CaseInsensitive ) )
+    if( dir.endsWith( DIR_SEP "VIDEO_TS", Qt::CaseInsensitive ) )
         scheme = "dvd";
-    else if( dir.endsWith( "BDMV", Qt::CaseInsensitive ) )
+    else if( dir.endsWith( DIR_SEP "BDMV", Qt::CaseInsensitive ) )
     {
         scheme = "bluray";
         dir.remove( "BDMV" );
@@ -524,20 +516,14 @@ static void openDirectory( intf_thread_t *p_intf, bool pl, bool go )
 
     char *uri = vlc_path2uri( qtu( toNativeSeparators( dir ) ), scheme );
     if( unlikely(uri == NULL) )
-        return;
+        return QString();
 
-    RecentsMRL::getInstance( p_intf )->addRecent( qfu(uri) );
-
-    input_item_t *p_input = input_item_New( uri, NULL );
+    dir = qfu( uri );
     free( uri );
-    if( unlikely( p_input == NULL ) )
-        return;
 
-    /* FIXME: playlist_AddInput() can fail */
-    playlist_AddInput( THEPL, p_input,
-                       go ? ( PLAYLIST_APPEND | PLAYLIST_GO ) : PLAYLIST_APPEND,
-                       PLAYLIST_END, pl, pl_Unlocked );
-    vlc_gc_decref( p_input );
+    RecentsMRL::getInstance( p_intf )->addRecent( dir );
+
+    return dir;
 }
 
 void DialogsProvider::PLOpenDir()
@@ -550,11 +536,6 @@ void DialogsProvider::PLAppendDir()
     openDirectory( p_intf, true, false );
 }
 
-void DialogsProvider::MLAppendDir()
-{
-    openDirectory( p_intf, false , false );
-}
-
 /****************
  * Playlist     *
  ****************/
@@ -568,7 +549,7 @@ void DialogsProvider::openAPlaylist()
     }
 }
 
-void DialogsProvider::saveAPlaylist()
+void DialogsProvider::saveAPlaylist(playlist_t *p_playlist, playlist_item_t *p_node)
 {
     static const struct
     {
@@ -634,12 +615,31 @@ void DialogsProvider::saveAPlaylist()
 
     if ( psz_selected_module )
     {
-        playlist_Export( THEPL, qtu( toNativeSeparators( file ) ),
-                         THEPL->p_playing, psz_selected_module );
+        playlist_Export( p_playlist, qtu( toNativeSeparators( file ) ),
+                         p_node, psz_selected_module );
         getSettings()->setValue( "last-playlist-ext", psz_last_playlist_ext );
     }
 }
 
+void DialogsProvider::savePlayingToPlaylist()
+{
+    saveAPlaylist(THEPL, THEPL->p_playing);
+}
+
+void DialogsProvider::saveRecentsToPlaylist()
+{
+    playlist_item_t *p_node_recents = RecentsMRL::getInstance(p_intf)->toPlaylist(0);
+
+    if (p_node_recents == NULL)
+    {
+        msg_Warn(p_intf, "cannot create playlist from recents");
+        return;
+    }
+
+    saveAPlaylist(THEPL, p_node_recents);
+    playlist_NodeDelete(THEPL, p_node_recents, true, false);
+}
+
 /****************************************************************************
  * Sout emulation
  ****************************************************************************/
@@ -685,29 +685,7 @@ void DialogsProvider::streamingDialog( QWidget *parent,
     {
         options += soutoption.split( " :");
 
-        /* Create Input */
-        input_item_t *p_input;
-        p_input = input_item_New( qtu( mrl ), _("Streaming") );
-
-        /* Add normal Options */
-        for( int j = 0; j < options.count(); j++ )
-        {
-            QString qs = colon_unescape( options[j] );
-            if( !qs.isEmpty() )
-            {
-                input_item_AddOption( p_input, qtu( qs ),
-                        VLC_INPUT_OPTION_TRUSTED );
-                msg_Dbg( p_intf, "Adding option: %s", qtu( qs ) );
-            }
-        }
-
-        /* Switch between enqueuing and starting the item */
-        /* FIXME: playlist_AddInput() can fail */
-        playlist_AddInput( THEPL, p_input,
-                PLAYLIST_APPEND | PLAYLIST_GO, PLAYLIST_END, true, pl_Unlocked );
-        vlc_gc_decref( p_input );
-
-        RecentsMRL::getInstance( p_intf )->addRecent( mrl );
+        Open::openMRLwithOptions( p_intf, mrl, &options, true, true, _("Streaming") );
     }
 }
 
@@ -750,8 +728,8 @@ void DialogsProvider::loadSubtitlesFile()
     free( path2 );
     foreach( const QString &qsFile, qsl )
     {
-        if( input_AddSubtitle( p_input, qtu( toNativeSeparators( qsFile ) ),
-                    true ) )
+        if( input_AddSubtitleOSD( p_input, qtu( toNativeSeparators( qsFile ) ),
+                    true, true ) )
             msg_Warn( p_intf, "unable to load subtitles from '%s'",
                       qtu( qsFile ) );
     }
@@ -782,12 +760,3 @@ void DialogsProvider::SDMenuAction( const QString& data )
         playlist_ServicesDiscoveryRemove( THEPL, qtu( data ) );
 }
 
-/**
- * Play the MRL contained in the Recently played menu.
- **/
-void DialogsProvider::playMRL( const QString &mrl )
-{
-    playlist_Add( THEPL, qtu(mrl), NULL,
-                  PLAYLIST_APPEND | PLAYLIST_GO , PLAYLIST_END, true, false );
-    RecentsMRL::getInstance( p_intf )->addRecent( mrl );
-}
