@@ -2,7 +2,7 @@
  * main_interface.cpp : Main interface
  ****************************************************************************
  * Copyright (C) 2006-2011 VideoLAN and AUTHORS
- * $Id: 5a13f50aa819c89349e80e886fb4e691ba574ffe $
+ * $Id: 8c911e87c7874ab93b84d878e3aaa528e7339704 $
  *
  * Authors: Clément Stenac <zorglub@videolan.org>
  *          Jean-Baptiste Kempf <jb@videolan.org>
@@ -60,9 +60,17 @@
 #include <QStackedWidget>
 #include <QFileInfo>
 
+#include <QTimer>
+
 #include <vlc_keys.h>                       /* Wheel event */
 #include <vlc_vout_display.h>               /* vout_thread_t and VOUT_ events */
 
+
+#if defined(_WIN32) && HAS_QT5
+#include <QWindow>
+#include <qpa/qplatformnativeinterface.h>
+#endif
+
 // #define DEBUG_INTF
 
 /* Callback prototypes */
@@ -112,7 +120,7 @@ MainInterface::MainInterface( intf_thread_t *_p_intf ) : QVLCMW( _p_intf )
     setWindowRole( "vlc-main" );
     setWindowIcon( QApplication::windowIcon() );
     setWindowOpacity( var_InheritFloat( p_intf, "qt-opacity" ) );
-#ifdef Q_WS_MAC
+#ifdef Q_OS_MAC
     setAttribute( Qt::WA_MacBrushedMetal );
 #endif
 
@@ -163,11 +171,6 @@ MainInterface::MainInterface( intf_thread_t *_p_intf ) : QVLCMW( _p_intf )
     createStatusBar();
     setStatusBarVisibility( getSettings()->value( "MainWindow/status-bar-visible", false ).toBool() );
 
-    /********************
-     * Input Manager    *
-     ********************/
-    MainInputManager::getInstance( p_intf );
-
 #ifdef _WIN32
     himl = NULL;
     p_taskbl = NULL;
@@ -365,6 +368,79 @@ void MainInterface::reloadPrefs()
     }
 }
 
+void MainInterface::createResumePanel( QWidget *w )
+{
+    resumePanel = new QWidget( w );
+    resumePanel->hide();
+    QHBoxLayout *resumePanelLayout = new QHBoxLayout( resumePanel );
+    resumePanelLayout->setSpacing( 0 ); resumePanelLayout->setMargin( 0 );
+
+    QLabel *continuePixmapLabel = new QLabel();
+    continuePixmapLabel->setPixmap( QPixmap( ":/menu/help" ) );
+    continuePixmapLabel->setContentsMargins( 5, 0, 5, 0 );
+
+    QLabel *continueLabel = new QLabel( qtr( "Do you want to restart the playback where left off?") );
+
+    QToolButton *cancel = new QToolButton( resumePanel );
+    cancel->setAutoRaise( true );
+    cancel->setText( "X" );
+
+    QPushButton *ok = new QPushButton( qtr( "&Continue" )  );
+
+    resumePanelLayout->addWidget( continuePixmapLabel );
+    resumePanelLayout->addWidget( continueLabel );
+    resumePanelLayout->addStretch( 1 );
+    resumePanelLayout->addWidget( ok );
+    resumePanelLayout->addWidget( cancel );
+
+    resumeTimer = new QTimer( resumePanel );
+    resumeTimer->setSingleShot( true );
+    resumeTimer->setInterval( 6000 );
+
+    CONNECT( resumeTimer, timeout(), this, hideResumePanel() );
+    CONNECT( cancel, clicked(), this, hideResumePanel() );
+    CONNECT( THEMIM->getIM(), resumePlayback(int64_t), this, showResumePanel(int64_t) );
+    BUTTONACT( ok, resumePlayback() );
+
+    w->layout()->addWidget( resumePanel );
+}
+
+void MainInterface::showResumePanel( int64_t _time ) {
+    int setting = var_InheritInteger( p_intf, "qt-continue" );
+
+    if( setting == 0 )
+        return;
+
+    i_resumeTime = _time;
+
+    if( setting == 2)
+        resumePlayback();
+    else
+    {
+        resumePanel->setVisible(true);
+        resumeTimer->start();
+    }
+}
+
+void MainInterface::hideResumePanel()
+{
+    if( resumePanel->isVisible() )
+    {
+        if( !isFullScreen() && !isMaximized() )
+            resize( width(), height() - resumePanel->height() );
+        resumePanel->hide();
+        resumeTimer->stop();
+    }
+}
+
+void MainInterface::resumePlayback()
+{
+    if( THEMIM->getIM()->hasInput() ) {
+        var_SetTime( THEMIM->getInput(), "time", i_resumeTime );
+    }
+    hideResumePanel();
+}
+
 void MainInterface::createMainWidget( QSettings *creationSettings )
 {
     /* Create the main Widget and the mainLayout */
@@ -374,6 +450,7 @@ void MainInterface::createMainWidget( QSettings *creationSettings )
     main->setContentsMargins( 0, 0, 0, 0 );
     mainLayout->setSpacing( 0 ); mainLayout->setMargin( 0 );
 
+    createResumePanel( main );
     /* */
     stackCentralW = new QVLCStackedWidget( main );
 
@@ -535,7 +612,7 @@ void MainInterface::debug()
 #endif
 }
 
-inline void MainInterface::showVideo() { showTab( videoWidget ); }
+inline void MainInterface::showVideo() { showTab( videoWidget ); setRaise(); }
 inline void MainInterface::restoreStackOldWidget()
             { showTab( stackCentralOldWidget ); }
 
@@ -634,8 +711,8 @@ void MainInterface::toggleFSC()
 
 /**
  * NOTE:
- * You must not change the state of this object or other Qt4 UI objects,
- * from the video output thread - only from the Qt4 UI main loop thread.
+ * You must not change the state of this object or other Qt UI objects,
+ * from the video output thread - only from the Qt UI main loop thread.
  * All window provider queries must be handled through signals or events.
  * That's why we have all those emit statements...
  */
@@ -685,11 +762,12 @@ void MainInterface::releaseVideoSlot( void )
 {
     /* This function is called when the embedded video window is destroyed,
      * or in the rare case that the embedded window is still here but the
-     * Qt4 interface exits. */
+     * Qt interface exits. */
     assert( videoWidget );
     videoWidget->release();
     setVideoOnTop( false );
     setVideoFullScreen( false );
+    hideResumePanel();
 
     if( stackCentralW->currentWidget() == videoWidget )
         restoreStackOldWidget();
@@ -754,6 +832,9 @@ void MainInterface::setVideoFullScreen( bool fs )
          * qt-fullscreen-screennumber is forced) */
         setMinimalView( b_minimalView );
         setInterfaceFullScreen( b_interfaceFullScreen );
+#ifdef _WIN32
+        changeThumbbarButtons( THEMIM->getIM()->playingStatus() );
+#endif
     }
     videoWidget->sync();
 }
@@ -971,8 +1052,10 @@ void MainInterface::setStatusBarVisibility( bool b_visible )
 
 void MainInterface::setPlaylistVisibility( bool b_visible )
 {
-    if ( !isPlDocked() && !THEDP->isDying() )
-        playlistVisible = b_visible;
+    if( isPlDocked() || THEDP->isDying() || (playlistWidget && playlistWidget->isMinimized() ) )
+        return;
+
+    playlistVisible = b_visible;
 }
 
 #if 0
@@ -1102,10 +1185,18 @@ void MainInterface::toggleUpdateSystrayMenu()
         /* check if any visible window is above vlc in the z-order,
          * but ignore the ones always on top
          * and the ones which can't be activated */
+        HWND winId;
+#if HAS_QT5
+        QWindow *window = windowHandle();
+        winId = static_cast<HWND>(QGuiApplication::platformNativeInterface()->nativeResourceForWindow("handle", window));
+#else
+        winId = internalWinId();
+#endif
+
         WINDOWINFO wi;
         HWND hwnd;
         wi.cbSize = sizeof( WINDOWINFO );
-        for( hwnd = GetNextWindow( internalWinId(), GW_HWNDPREV );
+        for( hwnd = GetNextWindow( winId, GW_HWNDPREV );
                 hwnd && ( !IsWindowVisible( hwnd ) ||
                     ( GetWindowInfo( hwnd, &wi ) &&
                       (wi.dwExStyle&WS_EX_NOACTIVATE) ) );
@@ -1121,7 +1212,7 @@ void MainInterface::toggleUpdateSystrayMenu()
             }
 #else
         hide();
-#endif
+#endif // _WIN32
     }
     if( sysTray )
         VLCMenuBar::updateSystrayMenu( this, p_intf );
@@ -1154,7 +1245,7 @@ void MainInterface::handleSystrayClick(
     {
         case QSystemTrayIcon::Trigger:
         case QSystemTrayIcon::DoubleClick:
-#ifdef Q_WS_MAC
+#ifdef Q_OS_MAC
             VLCMenuBar::updateSystrayMenu( this, p_intf );
 #else
             toggleUpdateSystrayMenu();
@@ -1276,9 +1367,9 @@ void MainInterface::dropEventPlay( QDropEvent *event, bool b_play, bool b_playli
     /* D&D of a subtitles file, add it on the fly */
     if( mimeData->urls().count() == 1 && THEMIM->getIM()->hasInput() )
     {
-        if( !input_AddSubtitle( THEMIM->getInput(),
+        if( !input_AddSubtitleOSD( THEMIM->getInput(),
                  qtu( toNativeSeparators( mimeData->urls()[0].toLocalFile() ) ),
-                 true ) )
+                 true, true ) )
         {
             event->accept();
             return;
@@ -1308,11 +1399,8 @@ void MainInterface::dropEventPlay( QDropEvent *event, bool b_play, bool b_playli
             }
             if( mrl.length() > 0 )
             {
-                playlist_Add( THEPL, qtu(mrl), NULL,
-                          PLAYLIST_APPEND | (first ? PLAYLIST_GO: PLAYLIST_PREPARSE),
-                          PLAYLIST_END, b_playlist, pl_Unlocked );
+                Open::openMRL( p_intf, mrl, first, b_playlist );
                 first = false;
-                RecentsMRL::getInstance( p_intf )->addRecent( mrl );
             }
         }
     }
@@ -1324,9 +1412,7 @@ void MainInterface::dropEventPlay( QDropEvent *event, bool b_play, bool b_playli
         QUrl(mimeData->text()).isValid() )
     {
         QString mrl = toURI( mimeData->text() );
-        playlist_Add( THEPL, qtu(mrl), NULL,
-                      PLAYLIST_APPEND | (first ? PLAYLIST_GO: PLAYLIST_PREPARSE),
-                      PLAYLIST_END, b_playlist, pl_Unlocked );
+        Open::openMRL( p_intf, mrl, first, b_playlist );
     }
     event->accept();
 }
