@@ -2,7 +2,7 @@
  * input_slider.hpp : VolumeSlider and SeekSlider
  ****************************************************************************
  * Copyright (C) 2006-2011 the VideoLAN team
- * $Id: 00b7b19daf5c48a10936a4d8b8943f1d2f8b48ae $
+ * $Id: 873b729993342badc538a76fd0f0cee34473aee9 $
  *
  * Authors: Clément Stenac <zorglub@videolan.org>
  *          Jean-Baptiste Kempf <jb@videolan.org>
@@ -30,12 +30,12 @@
 # include "config.h"
 #endif
 
-#include <vlc_common.h>
 #include "timetooltip.hpp"
 #include "styles/seekstyle.hpp"
 
 #include <QSlider>
 #include <QPainter>
+#include <QTime>
 
 #define MSTRTIME_MAX_SIZE 22
 
@@ -55,7 +55,7 @@ class SeekSlider : public QSlider
     Q_PROPERTY(qreal handleOpacity READ handleOpacity WRITE setHandleOpacity)
 public:
     SeekSlider( Qt::Orientation q, QWidget *_parent = 0, bool _classic = false );
-    ~SeekSlider();
+    virtual ~SeekSlider();
     void setChapters( SeekPoints * );
 
 protected:
@@ -66,13 +66,13 @@ protected:
     virtual void enterEvent( QEvent * );
     virtual void leaveEvent( QEvent * );
     virtual void hideEvent( QHideEvent * );
+    virtual void paintEvent(QPaintEvent *ev);
 
     virtual bool eventFilter( QObject *obj, QEvent *event );
 
     virtual QSize sizeHint() const;
 
     void processReleasedButton();
-    bool isAnimationRunning() const;
     qreal handleOpacity() const;
     void setHandleOpacity( qreal opacity );
     int handleLength();
@@ -85,6 +85,7 @@ private:
     QTimer *seekLimitTimer;
     TimeTooltip *mTimeTooltip;
     float f_buffering;
+    QTime bufferingStart;
     SeekPoints* chapters;
     bool b_classic;
     bool b_seekable;
@@ -118,8 +119,6 @@ private slots:
 signals:
     void sliderDragged( float );
 
-
-    friend class SeekStyle;
 };
 
 /* Sound Slider inherited directly from QAbstractSlider */
