@@ -2,7 +2,7 @@
  * theme_loader.cpp
  *****************************************************************************
  * Copyright (C) 2003 the VideoLAN team
- * $Id: e224364a1b8a7ef2dddeb9fa1d4aaaa618f58c66 $
+ * $Id: 6766be5a2ad68abb60f4f2567a8ad2d5085b7931 $
  *
  * Authors: Cyril Deguet     <asmax@via.ecp.fr>
  *          Olivier Teulière <ipkiss@via.ecp.fr>
@@ -28,9 +28,7 @@
 
 #include <fcntl.h>
 #include <sys/stat.h>
-#ifdef HAVE_UNISTD_H
-#   include <unistd.h>
-#endif
+#include <unistd.h>
 
 #include <vlc_common.h>
 #include <vlc_fs.h>
@@ -66,6 +64,9 @@ int makedir( const char *newdir );
 #define WINAMP2_XML_FILE "winamp2.xml"
 #define ZIP_BUFFER_SIZE 4096
 
+#ifndef O_BINARY
+#   define O_BINARY 0
+#endif
 
 bool ThemeLoader::load( const string &fileName )
 {
@@ -80,7 +81,7 @@ bool ThemeLoader::load( const string &fileName )
     // file...
 
 #if defined( HAVE_ZLIB_H )
-    if( ! extract( sToLocale( fileName ) ) && ! parse( path, fileName ) )
+    if( ! extract( fileName ) && ! parse( path, fileName ) )
         return false;
 #else
     if( ! parse( path, fileName ) )
@@ -117,6 +118,8 @@ bool ThemeLoader::extractTarGz( const string &tarFile, const string &rootDir )
     if( tar_open( &t, (char *)tarFile.c_str(), O_RDONLY ) == -1 )
 #endif
     {
+        msg_Dbg( getIntf(), "failed to open %s as a gzip tar file",
+                            tarFile.c_str() );
         return false;
     }
 
@@ -134,17 +137,40 @@ bool ThemeLoader::extractTarGz( const string &tarFile, const string &rootDir )
     return true;
 }
 
+static voidpf ZCALLBACK open_vlc( voidpf opaque, const char *filename, int mode)
+{
+    (void)mode;
+    intf_thread_t *pIntf = (intf_thread_t *)opaque;
+
+    FILE *stream = vlc_fopen( filename, "rb" );
+    if( stream == NULL )
+        msg_Dbg( pIntf, "vlc_fopen failed for %s", filename );
+    return stream;
+}
 
 bool ThemeLoader::extractZip( const string &zipFile, const string &rootDir )
 {
     bool b_isWsz = strstr( zipFile.c_str(), ".wsz" );
 
     // Try to open the ZIP file
-    unzFile file = unzOpen( zipFile.c_str() );
-    unz_global_info info;
+    zlib_filefunc_def descr;
+    fill_fopen_filefunc( &descr );
+    descr.zopen_file = open_vlc;
+    descr.opaque = getIntf();
 
+    unzFile file = unzOpen2( zipFile.c_str(), &descr );
+    if( file == 0 )
+    {
+        msg_Dbg( getIntf(), "failed to open %s as a zip file",
+                 zipFile.c_str() );
+        return false;
+    }
+    unz_global_info info;
     if( unzGetGlobalInfo( file, &info ) != UNZ_OK )
     {
+        msg_Dbg( getIntf(), "failed to read zip info from %s",
+                 zipFile.c_str() );
+        unzClose( file );
         return false;
     }
     // Extract all the files in the archive
@@ -161,7 +187,7 @@ bool ThemeLoader::extractZip( const string &zipFile, const string &rootDir )
         if( i < info.number_entry - 1 )
         {
             // Go the next file in the archive
-            if( unzGoToNextFile( file ) !=UNZ_OK )
+            if( unzGoToNextFile( file ) != UNZ_OK )
             {
                 msg_Warn( getIntf(), "error while unzipping %s",
                           zipFile.c_str() );
@@ -215,7 +241,7 @@ bool ThemeLoader::extractFileInZip( unzFile file, const string &rootDir,
             return false;
         }
         makedir( basePath.c_str() );
-        FILE *fout = fopen( fullPath.c_str(), "wb" );
+        FILE *fout = vlc_fopen( fullPath.c_str(), "wb" );
         if( fout == NULL )
         {
             msg_Err( getIntf(), "error opening %s", fullPath.c_str() );
@@ -265,9 +291,9 @@ bool ThemeLoader::extractFileInZip( unzFile file, const string &rootDir,
 bool ThemeLoader::extract( const string &fileName )
 {
     bool result = true;
-    char *tmpdir = tempnam( NULL, "vlt" );
-    string tempPath = sFromLocale( tmpdir );
-    free( tmpdir );
+    string tempPath = getTmpDir();
+    if( tempPath.empty() )
+        return false;
 
     // Extract the file in a temporary directory
     if( ! extractTarGz( fileName, tempPath ) &&
@@ -394,11 +420,10 @@ bool ThemeLoader::findFile( const string &rootDir, const string &rFileName,
     // Path separator
     const string &sep = OSFactory::instance( getIntf() )->getDirSeparator();
 
-    DIR *pCurrDir;
-    char *pszDirContent;
+    const char *pszDirContent;
 
     // Open the dir
-    pCurrDir = vlc_opendir( rootDir.c_str() );
+    DIR *pCurrDir = vlc_opendir( rootDir.c_str() );
 
     if( pCurrDir == NULL )
     {
@@ -430,7 +455,6 @@ bool ThemeLoader::findFile( const string &rootDir, const string &rFileName,
                 // Can we find the file in this subdirectory?
                 if( findFile( newURI, rFileName, themeFilePath ) )
                 {
-                    free( pszDirContent );
                     closedir( pCurrDir );
                     return true;
                 }
@@ -441,14 +465,11 @@ bool ThemeLoader::findFile( const string &rootDir, const string &rFileName,
                 if( rFileName == string( pszDirContent ) )
                 {
                     themeFilePath = newURI;
-                    free( pszDirContent );
                     closedir( pCurrDir );
                     return true;
                 }
             }
         }
-
-        free( pszDirContent );
     }
 
     closedir( pCurrDir );
@@ -498,14 +519,26 @@ int tar_open( TAR **t, char *pathname, int oflags )
 {
     (void)oflags;
 
-    gzFile f = gzopen( pathname, "rb" );
+    int fd = vlc_open( pathname, O_BINARY | O_RDONLY );
+    if( !fd )
+    {
+        fprintf( stderr, "Couldn't open %s\n", pathname );
+        return -1;
+    }
+    gzFile f = gzdopen( fd, "rb" );
     if( f == NULL )
     {
         fprintf( stderr, "Couldn't gzopen %s\n", pathname );
+        close( fd );
         return -1;
     }
 
     *t = (gzFile *)malloc( sizeof(gzFile) );
+    if( *t == NULL )
+    {
+        gzclose( f );
+        return -1;
+    }
     **t = f;
     return 0;
 }
@@ -570,7 +603,7 @@ int tar_extract_all( TAR *t, char *prefix )
                 remaining = getoct( buffer.header.size, 12 );
                 if( !remaining ) outfile = NULL; else
                 {
-                    outfile = fopen( fname, "wb" );
+                    outfile = vlc_fopen( fname, "wb" );
                     if( outfile == NULL )
                     {
                         /* try creating directory */
@@ -580,7 +613,7 @@ int tar_extract_all( TAR *t, char *prefix )
                             *p = '\0';
                             makedir( fname );
                             *p = '/';
-                            outfile = fopen( fname, "wb" );
+                            outfile = vlc_fopen( fname, "wb" );
                             if( !outfile )
                             {
                                 fprintf( stderr, "tar couldn't create %s\n",
@@ -610,7 +643,7 @@ int tar_extract_all( TAR *t, char *prefix )
                     fprintf( stderr, "error writing %s skipping...\n", fname );
                     fclose( outfile );
                     outfile = NULL;
-                    unlink( fname );
+                    vlc_unlink( fname );
                 }
             }
             remaining -= bytes;
@@ -732,11 +765,17 @@ int gzopen_frontend( const char *pathname, int oflags, int mode )
         errno = EINVAL;
         return -1;
     }
-
-    gzf = gzopen( pathname, gzflags );
+    int fd = vlc_open( pathname, oflags );
+    if( !fd )
+    {
+        fprintf( stderr, "Couldn't open %s\n", pathname );
+        return -1;
+    }
+    gzf = gzdopen( fd, gzflags );
     if( !gzf )
     {
         errno = ENOMEM;
+        close( fd );
         return -1;
     }
 
@@ -776,4 +815,32 @@ int gzwrite_frontend( int fd, const void * p_buffer, size_t i_length )
     return -1;
 }
 
+// FIXME: could become a skins2 OS factory function or a vlc core function
+string ThemeLoader::getTmpDir( )
+{
+#if defined( _WIN32 )
+    wchar_t *tmpdir = _wtempnam( NULL, L"vlt" );
+    if( tmpdir == NULL )
+        return "";
+    char* utf8 = FromWide( tmpdir );
+    free( tmpdir );
+    string tempPath( utf8 ? utf8 : "" );
+    free( utf8 );
+    return tempPath;
+
+#elif defined( __OS2__ )
+    char *tmpdir = tempnam( NULL, "vlt" );
+    if( tmpdir == NULL )
+        return "";
+    string tempPath( sFromLocale( tmpdir ));
+    free( tmpdir );
+    return tempPath;
+
+#else
+    char templ[] = "/tmp/vltXXXXXX";
+    char *tmpdir = mkdtemp( templ );
+    return string( tmpdir ? tmpdir : "");
+#endif
+}
+
 #endif
