@@ -1,7 +1,7 @@
 /*****************************************************************************
  * gnutls.c
  *****************************************************************************
- * Copyright (C) 2004-2012 Rémi Denis-Courmont
+ * Copyright (C) 2004-2015 Rémi Denis-Courmont
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -38,7 +38,6 @@
 
 #include <gnutls/gnutls.h>
 #include <gnutls/x509.h>
-#include "dhparams.h"
 
 /*****************************************************************************
  * Module descriptor
@@ -87,52 +86,65 @@ vlc_module_begin ()
             change_string_list (priorities_values, priorities_text)
 vlc_module_end ()
 
+#if (GNUTLS_VERSION_NUMBER >= 0x030300)
+static int gnutls_Init (vlc_object_t *obj)
+{
+    const char *version = gnutls_check_version ("3.3.0");
+    if (version == NULL)
+    {
+        msg_Err (obj, "unsupported GnuTLS version");
+        return -1;
+    }
+    msg_Dbg (obj, "using GnuTLS version %s", version);
+    return 0;
+}
+
+# define gnutls_Deinit() (void)0
+#else
+#define GNUTLS_SEC_PARAM_MEDIUM GNUTLS_SEC_PARAM_NORMAL
 static vlc_mutex_t gnutls_mutex = VLC_STATIC_MUTEX;
 
 /**
  * Initializes GnuTLS with proper locking.
  * @return VLC_SUCCESS on success, a VLC error code otherwise.
  */
-static int gnutls_Init (vlc_object_t *p_this)
+static int gnutls_Init (vlc_object_t *obj)
 {
-    int ret = VLC_EGENERIC;
-
-    vlc_mutex_lock (&gnutls_mutex);
-    if (gnutls_global_init ())
+    const char *version = gnutls_check_version ("3.0.20");
+    if (version == NULL)
     {
-        msg_Err (p_this, "cannot initialize GnuTLS");
-        goto error;
+        msg_Err (obj, "unsupported GnuTLS version");
+        return -1;
     }
+    msg_Dbg (obj, "using GnuTLS version %s", version);
 
-    const char *psz_version = gnutls_check_version ("3.0.20");
-    if (psz_version == NULL)
+    if (gnutls_check_version ("3.3.0") == NULL)
     {
-        msg_Err (p_this, "unsupported GnuTLS version");
-        gnutls_global_deinit ();
-        goto error;
+         int val;
+
+         vlc_mutex_lock (&gnutls_mutex);
+         val = gnutls_global_init ();
+         vlc_mutex_unlock (&gnutls_mutex);
+
+         if (val)
+         {
+             msg_Err (obj, "cannot initialize GnuTLS");
+             return -1;
+         }
     }
-
-    msg_Dbg (p_this, "GnuTLS v%s initialized", psz_version);
-    ret = VLC_SUCCESS;
-
-error:
-    vlc_mutex_unlock (&gnutls_mutex);
-    return ret;
+    return 0;
 }
 
-
 /**
  * Deinitializes GnuTLS.
  */
-static void gnutls_Deinit (vlc_object_t *p_this)
+static void gnutls_Deinit (void)
 {
     vlc_mutex_lock (&gnutls_mutex);
-
     gnutls_global_deinit ();
-    msg_Dbg (p_this, "GnuTLS deinitialized");
     vlc_mutex_unlock (&gnutls_mutex);
 }
-
+#endif
 
 static int gnutls_Error (vlc_object_t *obj, int val)
 {
@@ -378,6 +390,9 @@ static int gnutls_HandshakeAndValidate (vlc_tls_t *session, const char *host,
             return -1;
     }
 
+    if (host == NULL)
+        return status ? -1 : 0;
+
     /* certificate (host)name verification */
     const gnutls_datum_t *data;
     unsigned count;
@@ -389,8 +404,6 @@ static int gnutls_HandshakeAndValidate (vlc_tls_t *session, const char *host,
     }
     msg_Dbg (session, "%u certificate(s) in the list", count);
 
-    if (val || host == NULL)
-        return val;
     if (status && gnutls_CertSearch (session, host, service, data))
         return -1;
 
@@ -566,7 +579,8 @@ static int gnutls_AddCA (vlc_tls_creds_t *crd, const char *path)
     block_t *block = block_FilePath (path);
     if (block == NULL)
     {
-        msg_Err (crd, "cannot read trusted CA from %s: %m", path);
+        msg_Err (crd, "cannot read trusted CA from %s: %s", path,
+                 vlc_strerror_c(errno));
         return VLC_EGENERIC;
     }
 
@@ -605,7 +619,8 @@ static int gnutls_AddCRL (vlc_tls_creds_t *crd, const char *path)
     block_t *block = block_FilePath (path);
     if (block == NULL)
     {
-        msg_Err (crd, "cannot read CRL from %s: %m", path);
+        msg_Err (crd, "cannot read CRL from %s: %s", path,
+                 vlc_strerror_c(errno));
         return VLC_EGENERIC;
     }
 
@@ -639,7 +654,10 @@ static int OpenServer (vlc_tls_creds_t *crd, const char *cert, const char *key)
 
     vlc_tls_creds_sys_t *sys = malloc (sizeof (*sys));
     if (unlikely(sys == NULL))
-        goto error;
+    {
+        gnutls_Deinit ();
+        return VLC_ENOMEM;
+    }
 
     crd->sys     = sys;
     crd->add_CA  = gnutls_AddCA;
@@ -655,22 +673,26 @@ static int OpenServer (vlc_tls_creds_t *crd, const char *cert, const char *key)
     {
         msg_Err (crd, "cannot allocate credentials: %s",
                  gnutls_strerror (val));
-        goto error;
+        free (sys);
+        gnutls_Deinit ();
+        return VLC_ENOMEM;
     }
 
     block_t *certblock = block_FilePath (cert);
     if (certblock == NULL)
     {
-        msg_Err (crd, "cannot read certificate chain from %s: %m", cert);
-        return VLC_EGENERIC;
+        msg_Err (crd, "cannot read certificate chain from %s: %s", cert,
+                 vlc_strerror_c(errno));
+        goto error;
     }
 
     block_t *keyblock = block_FilePath (key);
     if (keyblock == NULL)
     {
-        msg_Err (crd, "cannot read private key from %s: %m", key);
+        msg_Err (crd, "cannot read private key from %s: %s", key,
+                 vlc_strerror_c(errno));
         block_Release (certblock);
-        return VLC_EGENERIC;
+        goto error;
     }
 
     gnutls_datum_t pub = {
@@ -688,23 +710,21 @@ static int OpenServer (vlc_tls_creds_t *crd, const char *cert, const char *key)
     if (val < 0)
     {
         msg_Err (crd, "cannot load X.509 key: %s", gnutls_strerror (val));
-        gnutls_certificate_free_credentials (sys->x509_cred);
         goto error;
     }
 
     /* FIXME:
+     * - regenerate these regularly
      * - support other cipher suites
      */
     val = gnutls_dh_params_init (&sys->dh_params);
     if (val >= 0)
     {
-        const gnutls_datum_t data = {
-            .data = (unsigned char *)dh_params,
-            .size = sizeof (dh_params) - 1,
-        };
+        gnutls_sec_param_t sec = GNUTLS_SEC_PARAM_MEDIUM;
+        unsigned bits = gnutls_sec_param_to_pk_bits (GNUTLS_PK_DH, sec);
 
-        val = gnutls_dh_params_import_pkcs3 (sys->dh_params, &data,
-                                             GNUTLS_X509_FMT_PEM);
+        msg_Dbg (crd, "generating Diffie-Hellman %u-bits parameters...", bits);
+        val = gnutls_dh_params_generate2 (sys->dh_params, bits);
         if (val == 0)
             gnutls_certificate_set_dh_params (sys->x509_cred,
                                               sys->dh_params);
@@ -715,11 +735,13 @@ static int OpenServer (vlc_tls_creds_t *crd, const char *cert, const char *key)
                  gnutls_strerror (val));
     }
 
+    msg_Dbg (crd, "ciphers parameters loaded");
     return VLC_SUCCESS;
 
 error:
+    gnutls_certificate_free_credentials (sys->x509_cred);
     free (sys);
-    gnutls_Deinit (VLC_OBJECT(crd));
+    gnutls_Deinit ();
     return VLC_EGENERIC;
 }
 
@@ -734,8 +756,7 @@ static void CloseServer (vlc_tls_creds_t *crd)
     gnutls_certificate_free_credentials (sys->x509_cred);
     gnutls_dh_params_deinit (sys->dh_params);
     free (sys);
-
-    gnutls_Deinit (VLC_OBJECT(crd));
+    gnutls_Deinit ();
 }
 
 /**
@@ -778,7 +799,7 @@ static int OpenClient (vlc_tls_creds_t *crd)
     return VLC_SUCCESS;
 error:
     free (sys);
-    gnutls_Deinit (VLC_OBJECT(crd));
+    gnutls_Deinit ();
     return VLC_EGENERIC;
 }
 
@@ -788,6 +809,5 @@ static void CloseClient (vlc_tls_creds_t *crd)
 
     gnutls_certificate_free_credentials (sys->x509_cred);
     free (sys);
-
-    gnutls_Deinit (VLC_OBJECT(crd));
+    gnutls_Deinit ();
 }
