@@ -2,7 +2,7 @@
  * upnp.cpp :  UPnP discovery module (libupnp)
  *****************************************************************************
  * Copyright (C) 2004-2011 the VideoLAN team
- * $Id: 66223fa091369d81af1e0579b1606be2848dc761 $
+ * $Id: b5f74097fb10fd3f1a4b689044e8c7e4c20d2fd0 $
  *
  * Authors: Rémi Denis-Courmont <rem # videolan.org> (original plugin)
  *          Christian Henz <henz # c-lab.de>
@@ -32,7 +32,7 @@
 # include "config.h"
 #endif
 
-#include "upnp.hpp"
+#include "services_discovery/upnp.hpp"
 
 #include <vlc_plugin.h>
 #include <vlc_services_discovery.h>
@@ -110,7 +110,7 @@ static int Open( vlc_object_t *p_this )
     if( !( p_sd->p_sys = p_sys ) )
         return VLC_ENOMEM;
 
-#ifdef UPNP_ENABLE_IPV6
+#if defined( UPNP_ENABLE_IPV6 ) && !defined(_WIN32)
     char* psz_miface;
     psz_miface = var_InheritString( p_sd, "miface" );
     msg_Info( p_sd, "Initializing libupnp on '%s' interface", psz_miface );
@@ -822,7 +822,8 @@ bool MediaServer::_fetchContents( Container* p_parent, int i_offset )
     IXML_Document* p_response = _browseAction( p_parent->getObjectID(),
                                       "BrowseDirectChildren",
                                       "id,dc:title,res," /* Filter */
-                                      "sec:CaptionInfo,sec:CaptionInfoEx",
+                                      "sec:CaptionInfo,sec:CaptionInfoEx,"
+                                      "pv:subtitlefile",
                                       psz_starting_index, /* StartingIndex */
                                       "0", /* RequestedCount */
                                       "" /* SortCriteria */
@@ -912,6 +913,10 @@ bool MediaServer::_fetchContents( Container* p_parent, int i_offset )
                 psz_subtitles = xml_getChildElementValue( itemElement,
                         "sec:CaptionInfoEx" );
 
+            if ( !psz_subtitles )
+                psz_subtitles = xml_getChildElementValue( itemElement,
+                        "pv:subtitlefile" );
+
             /* Try to extract all resources in DIDL */
             IXML_NodeList* p_resource_list = ixmlDocument_getElementsByTagName( (IXML_Document*) itemElement, "res" );
             if ( p_resource_list )
