@@ -7,19 +7,19 @@
  * Authors: Christopher Mueller <christopher.mueller@itec.uni-klu.ac.at>
  *          Christian Timmerer  <christian.timmerer@itec.uni-klu.ac.at>
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU Lesser General Public License as published
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU Lesser General Public License as published
  * by the Free Software Foundation; either version 2.1 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
+ * along with this program; if not, write to the Free Software Foundation,
+ * Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
  *****************************************************************************/
 #ifdef HAVE_CONFIG_H
 # include "config.h"
@@ -31,9 +31,9 @@ using namespace dash::logic;
 using namespace dash::xml;
 using namespace dash::http;
 using namespace dash::mpd;
-using namespace dash::exception;
 
-AlwaysBestAdaptationLogic::AlwaysBestAdaptationLogic    (IMPDManager *mpdManager) : AbstractAdaptationLogic(mpdManager)
+AlwaysBestAdaptationLogic::AlwaysBestAdaptationLogic    (IMPDManager *mpdManager, stream_t *stream) :
+                           AbstractAdaptationLogic      (mpdManager, stream)
 {
     this->mpdManager    = mpdManager;
     this->count         = 0;
@@ -43,26 +43,31 @@ AlwaysBestAdaptationLogic::~AlwaysBestAdaptationLogic   ()
 {
 }
 
-Chunk*  AlwaysBestAdaptationLogic::getNextChunk() throw(EOFException)
+Chunk*  AlwaysBestAdaptationLogic::getNextChunk()
 {
     if(this->schedule.size() == 0)
-        throw EOFException();
+        return NULL;
 
     if(this->count == this->schedule.size())
-        throw EOFException();
+        return NULL;
 
-    for(size_t i = 0; i < this->schedule.size(); i++)
+    if ( this->count < this->schedule.size() )
     {
-        if(this->count == i)
-        {
-            Chunk *chunk = new Chunk();
-            chunk->setUrl(this->schedule.at(i)->getSourceUrl());
-            this->count++;
-            return chunk;
-        }
+        Chunk *chunk = new Chunk();
+        chunk->setUrl(this->schedule.at( this->count )->getSourceUrl());
+        this->count++;
+        return chunk;
     }
     return NULL;
 }
+
+const Representation *AlwaysBestAdaptationLogic::getCurrentRepresentation() const
+{
+    if ( this->count < this->schedule.size() )
+        return this->schedule.at( this->count )->getParentRepresentation();
+    return NULL;
+}
+
 void    AlwaysBestAdaptationLogic::initSchedule ()
 {
     if(this->mpdManager != NULL)
