@@ -7,19 +7,19 @@
  * Authors: Christopher Mueller <christopher.mueller@itec.uni-klu.ac.at>
  *          Christian Timmerer  <christian.timmerer@itec.uni-klu.ac.at>
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU Lesser General Public License as published
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU Lesser General Public License as published
  * by the Free Software Foundation; either version 2.1 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
+ * along with this program; if not, write to the Free Software Foundation,
+ * Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
  *****************************************************************************/
 #ifdef HAVE_CONFIG_H
 # include "config.h"
@@ -66,8 +66,7 @@ BasicCMParser::~BasicCMParser   ()
 
 bool    BasicCMParser::parse                ()
 {
-    this->setMPD();
-    return true;
+    return this->setMPD();
 }
 bool    BasicCMParser::setMPD()
 {
@@ -75,12 +74,6 @@ bool    BasicCMParser::setMPD()
     this->mpd = new MPD;
 
     std::map<std::string, std::string>::const_iterator  it;
-    it = attr.find( "profile" );
-    if ( it == attr.end() )
-        it = attr.find( "profiles" ); //The standard spells it the two ways...
-    if ( it != attr.end() )
-        this->mpd->setProfile( it->second );
-
     it = attr.find("mediaPresentationDuration");
     /*
         Standard specifies a default of "On-Demand",
@@ -161,7 +154,7 @@ void    BasicCMParser::setPeriods           (Node *root)
     for(size_t i = 0; i < periods.size(); i++)
     {
         Period *period = new Period();
-        this->setGroups(periods.at(i), period);
+        this->setAdaptationSet(periods.at(i), period);
         this->mpd->addPeriod(period);
     }
 }
@@ -237,7 +230,7 @@ void BasicCMParser::parseSegmentInfoCommon(Node *node, SegmentInfoCommon *segmen
     this->parseSegmentTimeline( node, segmentInfo );
 }
 
-void BasicCMParser::parseSegmentInfoDefault(Node *node, Group *group)
+void BasicCMParser::parseSegmentInfoDefault(Node *node, AdaptationSet *group)
 {
     Node*   segmentInfoDefaultNode = DOMHelper::getFirstChildElementByName( node, "SegmentInfoDefault" );
 
@@ -250,25 +243,27 @@ void BasicCMParser::parseSegmentInfoDefault(Node *node, Group *group)
     }
 }
 
-void    BasicCMParser::setGroups            (Node *root, Period *period)
+void    BasicCMParser::setAdaptationSet            (Node *root, Period *period)
 {
-    std::vector<Node *> groups = DOMHelper::getElementByTagName(root, "Group", false);
+    std::vector<Node *> adaptSets = DOMHelper::getElementByTagName(root, "AdaptationSet", false);
+    if ( adaptSets.size() == 0 ) //In some old file, AdaptationSet may still be called Group
+        adaptSets = DOMHelper::getElementByTagName(root, "Group", false);
 
-    for(size_t i = 0; i < groups.size(); i++)
+    for(size_t i = 0; i < adaptSets.size(); i++)
     {
-        const std::map<std::string, std::string>    attr = groups.at(i)->getAttributes();
-        Group *group = new Group;
-        if ( this->parseCommonAttributesElements( groups.at( i ), group, NULL ) == false )
+        const std::map<std::string, std::string>    attr = adaptSets.at(i)->getAttributes();
+        AdaptationSet *adaptSet = new AdaptationSet;
+        if ( this->parseCommonAttributesElements( adaptSets.at( i ), adaptSet, NULL ) == false )
         {
-            delete group;
+            delete adaptSet;
             continue ;
         }
         std::map<std::string, std::string>::const_iterator  it = attr.find( "subsegmentAlignmentFlag" );
         if ( it != attr.end() && it->second == "true" )
-            group->setSubsegmentAlignmentFlag( true ); //Otherwise it is false by default.
-        this->parseSegmentInfoDefault( groups.at( i ), group );
-        this->setRepresentations(groups.at(i), group);
-        period->addGroup(group);
+            adaptSet->setSubsegmentAlignmentFlag( true ); //Otherwise it is false by default.
+        this->parseSegmentInfoDefault( adaptSets.at( i ), adaptSet );
+        this->setRepresentations(adaptSets.at(i), adaptSet);
+        period->addAdaptationSet(adaptSet);
     }
 }
 
@@ -291,7 +286,7 @@ void BasicCMParser::parseTrickMode(Node *node, Representation *repr)
     repr->setTrickMode( trickMode );
 }
 
-void    BasicCMParser::setRepresentations   (Node *root, Group *group)
+void    BasicCMParser::setRepresentations   (Node *root, AdaptationSet *group)
 {
     std::vector<Node *> representations = DOMHelper::getElementByTagName(root, "Representation", false);
 
@@ -341,7 +336,7 @@ void    BasicCMParser::setRepresentations   (Node *root, Group *group)
     }
 }
 
-void    BasicCMParser::handleDependencyId( Representation *rep, const Group *group, const std::string &dependencyId )
+void    BasicCMParser::handleDependencyId(Representation *rep, const AdaptationSet *adaptationSet, const std::string &dependencyId )
 {
     if ( dependencyId.empty() == true )
         return ;
@@ -350,7 +345,7 @@ void    BasicCMParser::handleDependencyId( Representation *rep, const Group *gro
     {
         std::string     id;
         s >> id;
-        const Representation    *dep = group->getRepresentationById( id );
+        const Representation    *dep = adaptationSet->getRepresentationById( id );
         if ( dep )
             rep->addDependency( dep );
     }
@@ -404,7 +399,7 @@ Segment*    BasicCMParser::parseSegment( Node* node )
             seg = new SegmentTemplate( runtimeToken, this->currentRepresentation );
         }
         else
-            seg = new Segment;
+            seg = new Segment( this->currentRepresentation );
         if ( url.find( this->p_stream->psz_access ) != 0 ) //Relative url
             url = this->url + url;
         seg->setSourceUrl( url );
@@ -501,12 +496,6 @@ bool    BasicCMParser::resolveUrlTemplates( std::string &url, bool &containRunti
         }
         else if ( token == "$Bandwidth$" )
         {
-            if ( this->currentRepresentation->getBandwidth() < 0 )
-            {
-                std::cerr << "Representation doesn't have a valid bandwidth. "
-                             "Can't substitute tag $Bandwidth$" << std::endl;
-                return false;
-            }
             std::ostringstream  oss;
             oss << this->currentRepresentation->getBandwidth();
             url.replace( it, token.length(), oss.str() );
@@ -572,13 +561,14 @@ bool    BasicCMParser::parseCommonAttributesElements( Node *node, CommonAttribut
     {
         if ( parent && parent->getMimeType().empty() == false )
             common->setMimeType( parent->getMimeType() );
-        else
+        else if ( node->getName().find( "Representation" ) != std::string::npos )
         {
             std::cerr << "Missing mandatory attribute: @mimeType" << std::endl;
             return false;
         }
     }
-    common->setMimeType( it->second );
+    else
+        common->setMimeType( it->second );
     //Everything else is optionnal.
     it = attr.find( "width" );
     if ( it != attr.end() )
