@@ -1,8 +1,8 @@
 /*****************************************************************************
  * autodel.c: monitor mux inputs and automatically add/delete streams
  *****************************************************************************
- * Copyright (C) 2006 the VideoLAN team
- * $Id: 390af0174d571579f1f702e3fc014b3b279c3404 $
+ * Copyright (C) 2006 VLC authors and VideoLAN
+ * $Id: 734f91c5331dec573b57e6bc5e1c1da60cae11b6 $
  *
  * Authors: Christophe Massiot <massiot@via.ecp.fr>
  *
@@ -54,13 +54,13 @@ vlc_module_end ()
 /*****************************************************************************
  * Local prototypes
  *****************************************************************************/
-static sout_stream_id_t *Add   ( sout_stream_t *, es_format_t * );
-static int               Del   ( sout_stream_t *, sout_stream_id_t * );
-static int               Send  ( sout_stream_t *, sout_stream_id_t *, block_t * );
+static sout_stream_id_sys_t *Add   ( sout_stream_t *, es_format_t * );
+static int               Del   ( sout_stream_t *, sout_stream_id_sys_t * );
+static int               Send  ( sout_stream_t *, sout_stream_id_sys_t *, block_t * );
 
-struct sout_stream_id_t
+struct sout_stream_id_sys_t
 {
-    sout_stream_id_t *id;
+    sout_stream_id_sys_t *id;
     es_format_t fmt;
     mtime_t i_last;
     bool b_error;
@@ -68,7 +68,7 @@ struct sout_stream_id_t
 
 struct sout_stream_sys_t
 {
-    sout_stream_id_t **pp_es;
+    sout_stream_id_sys_t **pp_es;
     int i_es_num;
 };
 
@@ -111,10 +111,10 @@ static void Close( vlc_object_t * p_this )
     free( p_sys );
 }
 
-static sout_stream_id_t * Add( sout_stream_t *p_stream, es_format_t *p_fmt )
+static sout_stream_id_sys_t * Add( sout_stream_t *p_stream, es_format_t *p_fmt )
 {
     sout_stream_sys_t *p_sys = (sout_stream_sys_t *)p_stream->p_sys;
-    sout_stream_id_t *p_es = malloc( sizeof(sout_stream_id_t) );
+    sout_stream_id_sys_t *p_es = malloc( sizeof(sout_stream_id_sys_t) );
 
     p_es->fmt = *p_fmt;
     p_es->id = NULL;
@@ -125,10 +125,10 @@ static sout_stream_id_t * Add( sout_stream_t *p_stream, es_format_t *p_fmt )
     return p_es;
 }
 
-static int Del( sout_stream_t *p_stream, sout_stream_id_t *p_es )
+static int Del( sout_stream_t *p_stream, sout_stream_id_sys_t *p_es )
 {
     sout_stream_sys_t *p_sys = (sout_stream_sys_t *)p_stream->p_sys;
-    sout_stream_id_t *id = p_es->id;
+    sout_stream_id_sys_t *id = p_es->id;
 
     TAB_REMOVE( p_sys->i_es_num, p_sys->pp_es, p_es );
     free( p_es );
@@ -139,7 +139,7 @@ static int Del( sout_stream_t *p_stream, sout_stream_id_t *p_es )
         return VLC_SUCCESS;
 }
 
-static int Send( sout_stream_t *p_stream, sout_stream_id_t *p_es,
+static int Send( sout_stream_t *p_stream, sout_stream_id_sys_t *p_es,
                  block_t *p_buffer )
 {
     sout_stream_sys_t *p_sys = (sout_stream_sys_t *)p_stream->p_sys;
