@@ -1,24 +1,24 @@
 /*****************************************************************************
  * smem.c: stream output to memory buffer module
  *****************************************************************************
- * Copyright (C) 2009 the VideoLAN team
- * $Id: f4d3df8d5cdf5a26596a79270b8881f2362c74f1 $
+ * Copyright (C) 2009 VLC authors and VideoLAN
+ * $Id: 791f9f86aa20e2d080cdcf0b7d047c40f9f41a23 $
  *
  * Authors: Christophe Courtaut <christophe.courtaut@gmail.com>
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU Lesser General Public License as published by
+ * the Free Software Foundation; either version 2.1 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
+ * You should have received a copy of the GNU Lesser General Public License
+ * along with this program; if not, write to the Free Software Foundation,
+ * Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
  *****************************************************************************/
 
 /*****************************************************************************
@@ -125,19 +125,19 @@ static const char *const ppsz_sout_options[] = {
     "video-postrender-callback", "audio-postrender-callback", "video-data", "audio-data", "time-sync", NULL
 };
 
-static sout_stream_id_t *Add ( sout_stream_t *, es_format_t * );
-static int               Del ( sout_stream_t *, sout_stream_id_t * );
-static int               Send( sout_stream_t *, sout_stream_id_t *, block_t* );
+static sout_stream_id_sys_t *Add ( sout_stream_t *, es_format_t * );
+static int               Del ( sout_stream_t *, sout_stream_id_sys_t * );
+static int               Send( sout_stream_t *, sout_stream_id_sys_t *, block_t* );
 
-static sout_stream_id_t *AddVideo( sout_stream_t *p_stream, es_format_t *p_fmt );
-static sout_stream_id_t *AddAudio( sout_stream_t *p_stream, es_format_t *p_fmt );
+static sout_stream_id_sys_t *AddVideo( sout_stream_t *p_stream, es_format_t *p_fmt );
+static sout_stream_id_sys_t *AddAudio( sout_stream_t *p_stream, es_format_t *p_fmt );
 
-static int SendVideo( sout_stream_t *p_stream, sout_stream_id_t *id,
+static int SendVideo( sout_stream_t *p_stream, sout_stream_id_sys_t *id,
                       block_t *p_buffer );
-static int SendAudio( sout_stream_t *p_stream, sout_stream_id_t *id,
+static int SendAudio( sout_stream_t *p_stream, sout_stream_id_sys_t *id,
                       block_t *p_buffer );
 
-struct sout_stream_id_t
+struct sout_stream_id_sys_t
 {
     es_format_t* format;
     void *p_data;
@@ -146,10 +146,10 @@ struct sout_stream_id_t
 struct sout_stream_sys_t
 {
     vlc_mutex_t *p_lock;
-    void ( *pf_video_prerender_callback ) ( void* p_video_data, uint8_t** pp_pixel_buffer , int size );
-    void ( *pf_audio_prerender_callback ) ( void* p_audio_data, uint8_t** pp_pcm_buffer , unsigned int size );
-    void ( *pf_video_postrender_callback ) ( void* p_video_data, uint8_t* p_pixel_buffer, int width, int height, int pixel_pitch, int size, mtime_t pts );
-    void ( *pf_audio_postrender_callback ) ( void* p_audio_data, uint8_t* p_pcm_buffer, unsigned int channels, unsigned int rate, unsigned int nb_samples, unsigned int bits_per_sample, unsigned int size, mtime_t pts );
+    void ( *pf_video_prerender_callback ) ( void* p_video_data, uint8_t** pp_pixel_buffer, size_t size );
+    void ( *pf_audio_prerender_callback ) ( void* p_audio_data, uint8_t** pp_pcm_buffer, size_t size );
+    void ( *pf_video_postrender_callback ) ( void* p_video_data, uint8_t* p_pixel_buffer, int width, int height, int pixel_pitch, size_t size, mtime_t pts );
+    void ( *pf_audio_postrender_callback ) ( void* p_audio_data, uint8_t* p_pcm_buffer, unsigned int channels, unsigned int rate, unsigned int nb_samples, unsigned int bits_per_sample, size_t size, mtime_t pts );
     bool time_sync;
 };
 
@@ -173,19 +173,19 @@ static int Open( vlc_object_t *p_this )
     p_sys->time_sync = var_GetBool( p_stream, SOUT_CFG_PREFIX "time-sync" );
 
     psz_tmp = var_GetString( p_stream, SOUT_PREFIX_VIDEO "prerender-callback" );
-    p_sys->pf_video_prerender_callback = (void (*) (void *, uint8_t**, int))(intptr_t)atoll( psz_tmp );
+    p_sys->pf_video_prerender_callback = (void (*) (void *, uint8_t**, size_t))(intptr_t)atoll( psz_tmp );
     free( psz_tmp );
 
     psz_tmp = var_GetString( p_stream, SOUT_PREFIX_AUDIO "prerender-callback" );
-    p_sys->pf_audio_prerender_callback = (void (*) (void* , uint8_t**, unsigned int))(intptr_t)atoll( psz_tmp );
+    p_sys->pf_audio_prerender_callback = (void (*) (void* , uint8_t**, size_t))(intptr_t)atoll( psz_tmp );
     free( psz_tmp );
 
     psz_tmp = var_GetString( p_stream, SOUT_PREFIX_VIDEO "postrender-callback" );
-    p_sys->pf_video_postrender_callback = (void (*) (void*, uint8_t*, int, int, int, int, mtime_t))(intptr_t)atoll( psz_tmp );
+    p_sys->pf_video_postrender_callback = (void (*) (void*, uint8_t*, int, int, int, size_t, mtime_t))(intptr_t)atoll( psz_tmp );
     free( psz_tmp );
 
     psz_tmp = var_GetString( p_stream, SOUT_PREFIX_AUDIO "postrender-callback" );
-    p_sys->pf_audio_postrender_callback = (void (*) (void*, uint8_t*, unsigned int, unsigned int, unsigned int, unsigned int, unsigned int, mtime_t))(intptr_t)atoll( psz_tmp );
+    p_sys->pf_audio_postrender_callback = (void (*) (void*, uint8_t*, unsigned int, unsigned int, unsigned int, unsigned int, size_t, mtime_t))(intptr_t)atoll( psz_tmp );
     free( psz_tmp );
 
     /* Setting stream out module callbacks */
@@ -206,9 +206,9 @@ static void Close( vlc_object_t * p_this )
     free( p_stream->p_sys );
 }
 
-static sout_stream_id_t *Add( sout_stream_t *p_stream, es_format_t *p_fmt )
+static sout_stream_id_sys_t *Add( sout_stream_t *p_stream, es_format_t *p_fmt )
 {
-    sout_stream_id_t *id = NULL;
+    sout_stream_id_sys_t *id = NULL;
 
     if ( p_fmt->i_cat == VIDEO_ES )
         id = AddVideo( p_stream, p_fmt );
@@ -217,16 +217,17 @@ static sout_stream_id_t *Add( sout_stream_t *p_stream, es_format_t *p_fmt )
     return id;
 }
 
-static sout_stream_id_t *AddVideo( sout_stream_t *p_stream, es_format_t *p_fmt )
+static sout_stream_id_sys_t *AddVideo( sout_stream_t *p_stream, es_format_t *p_fmt )
 {
     char* psz_tmp;
-    sout_stream_id_t    *id;
+    sout_stream_id_sys_t    *id;
     int i_bits_per_pixel;
 
     switch( p_fmt->i_codec )
     {
         case VLC_CODEC_RGB32:
         case VLC_CODEC_RGBA:
+        case VLC_CODEC_ARGB:
             i_bits_per_pixel = 32;
             break;
         case VLC_CODEC_I444:
@@ -252,7 +253,7 @@ static sout_stream_id_t *AddVideo( sout_stream_t *p_stream, es_format_t *p_fmt )
             break;
     }
 
-    id = calloc( 1, sizeof( sout_stream_id_t ) );
+    id = calloc( 1, sizeof( sout_stream_id_sys_t ) );
     if( !id )
         return NULL;
 
@@ -265,10 +266,10 @@ static sout_stream_id_t *AddVideo( sout_stream_t *p_stream, es_format_t *p_fmt )
     return id;
 }
 
-static sout_stream_id_t *AddAudio( sout_stream_t *p_stream, es_format_t *p_fmt )
+static sout_stream_id_sys_t *AddAudio( sout_stream_t *p_stream, es_format_t *p_fmt )
 {
     char* psz_tmp;
-    sout_stream_id_t* id;
+    sout_stream_id_sys_t* id;
     int i_bits_per_sample = aout_BitsPerSample( p_fmt->i_codec );
 
     if( !i_bits_per_sample )
@@ -277,7 +278,7 @@ static sout_stream_id_t *AddAudio( sout_stream_t *p_stream, es_format_t *p_fmt )
         return NULL;
     }
 
-    id = calloc( 1, sizeof( sout_stream_id_t ) );
+    id = calloc( 1, sizeof( sout_stream_id_sys_t ) );
     if( !id )
         return NULL;
 
@@ -290,14 +291,14 @@ static sout_stream_id_t *AddAudio( sout_stream_t *p_stream, es_format_t *p_fmt )
     return id;
 }
 
-static int Del( sout_stream_t *p_stream, sout_stream_id_t *id )
+static int Del( sout_stream_t *p_stream, sout_stream_id_sys_t *id )
 {
     VLC_UNUSED( p_stream );
     free( id );
     return VLC_SUCCESS;
 }
 
-static int Send( sout_stream_t *p_stream, sout_stream_id_t *id,
+static int Send( sout_stream_t *p_stream, sout_stream_id_sys_t *id,
                  block_t *p_buffer )
 {
     if ( id->format->i_cat == VIDEO_ES )
@@ -307,26 +308,15 @@ static int Send( sout_stream_t *p_stream, sout_stream_id_t *id,
     return VLC_SUCCESS;
 }
 
-static int SendVideo( sout_stream_t *p_stream, sout_stream_id_t *id,
+static int SendVideo( sout_stream_t *p_stream, sout_stream_id_sys_t *id,
                       block_t *p_buffer )
 {
     sout_stream_sys_t *p_sys = p_stream->p_sys;
-    int i_line, i_line_size, i_size, i_pixel_pitch;
+    size_t i_size = p_buffer->i_buffer;
     uint8_t* p_pixels = NULL;
 
-    if( id->format->video.i_bits_per_pixel > 0 )
-    {
-        i_line = id->format->video.i_height;
-        i_pixel_pitch = id->format->video.i_bits_per_pixel / 8;
-        i_line_size = i_pixel_pitch * id->format->video.i_width;
-        i_size = i_line * i_line_size;
-    }
-    else
-    {
-        i_size = p_buffer->i_buffer;
-    }
     /* Calling the prerender callback to get user buffer */
-    p_sys->pf_video_prerender_callback( id->p_data, &p_pixels , i_size );
+    p_sys->pf_video_prerender_callback( id->p_data, &p_pixels, i_size );
 
     if (!p_pixels)
     {
@@ -336,22 +326,7 @@ static int SendVideo( sout_stream_t *p_stream, sout_stream_id_t *id,
     }
 
     /* Copying data into user buffer */
-    if( id->format->video.i_bits_per_pixel > 0 )
-    {
-        uint8_t *p_in = p_buffer->p_buffer;
-        uint8_t *p_out = p_pixels;
-
-        for ( int line = 0; line < i_line; line++ )
-        {
-            memcpy( p_out, p_in, i_line_size );
-            p_out += i_line_size;
-            p_in += i_line_size;
-        }
-    }
-    else
-    {
-        memcpy( p_pixels, p_buffer->p_buffer, i_size );
-    }
+    memcpy( p_pixels, p_buffer->p_buffer, i_size );
     /* Calling the postrender callback to tell the user his buffer is ready */
     p_sys->pf_video_postrender_callback( id->p_data, p_pixels,
                                          id->format->video.i_width, id->format->video.i_height,
@@ -360,7 +335,7 @@ static int SendVideo( sout_stream_t *p_stream, sout_stream_id_t *id,
     return VLC_SUCCESS;
 }
 
-static int SendAudio( sout_stream_t *p_stream, sout_stream_id_t *id,
+static int SendAudio( sout_stream_t *p_stream, sout_stream_id_sys_t *id,
                       block_t *p_buffer )
 {
     sout_stream_sys_t *p_sys = p_stream->p_sys;
