@@ -2,7 +2,7 @@
  * hqdn3d.c : high-quality denoise 3D ported from MPlayer
  *****************************************************************************
  * Copyright (C) 2011 VLC authors and VideoLAN
- * $Id: a78d66f72d108c864cfac057228c1e02dcbd2dbc $
+ * $Id: 44f93408371fefa42fb3d7583b4aa3740c3dadcd $
  *
  * Authors: Cheng Sun <chengsun9@gmail.com>
  *
@@ -43,6 +43,9 @@
 static int  Open         (vlc_object_t *);
 static void Close        (vlc_object_t *);
 static picture_t *Filter (filter_t *, picture_t *);
+static int DenoiseCallback( vlc_object_t *p_this, char const *psz_var,
+                            vlc_value_t oldval, vlc_value_t newval,
+                            void *p_data );
 
 /*****************************************************************************
  * Module descriptor
@@ -51,13 +54,9 @@ static picture_t *Filter (filter_t *, picture_t *);
 #define FILTER_PREFIX       "hqdn3d-"
 
 #define LUMA_SPAT_TEXT          N_("Spatial luma strength (0-254)")
-#define LUMA_SPAT_LONGTEXT      N_("Spatial luma strength (default 4)")
 #define CHROMA_SPAT_TEXT        N_("Spatial chroma strength (0-254)")
-#define CHROMA_SPAT_LONGTEXT    N_("Spatial chroma strength (default 3)")
 #define LUMA_TEMP_TEXT          N_("Temporal luma strength (0-254)")
-#define LUMA_TEMP_LONGTEXT      N_("Temporal luma strength (default 6)")
 #define CHROMA_TEMP_TEXT        N_("Temporal chroma strength (0-254)")
-#define CHROMA_TEMP_LONGTEXT    N_("Temporal chroma strength (default 4.5)")
 
 vlc_module_begin()
     set_shortname(N_("HQ Denoiser 3D"))
@@ -67,13 +66,13 @@ vlc_module_begin()
     set_subcategory(SUBCAT_VIDEO_VFILTER)
 
     add_float_with_range(FILTER_PREFIX "luma-spat", 4.0, 0.0, 254.0,
-            LUMA_SPAT_TEXT, LUMA_SPAT_LONGTEXT, false)
+            LUMA_SPAT_TEXT, LUMA_SPAT_TEXT, false)
     add_float_with_range(FILTER_PREFIX "chroma-spat", 3.0, 0.0, 254.0,
-            CHROMA_SPAT_TEXT, CHROMA_SPAT_LONGTEXT, false)
+            CHROMA_SPAT_TEXT, CHROMA_SPAT_TEXT, false)
     add_float_with_range(FILTER_PREFIX "luma-temp", 6.0, 0.0, 254.0,
-            LUMA_TEMP_TEXT, LUMA_TEMP_LONGTEXT, false)
+            LUMA_TEMP_TEXT, LUMA_TEMP_TEXT, false)
     add_float_with_range(FILTER_PREFIX "chroma-temp", 4.5, 0.0, 254.0,
-            CHROMA_TEMP_TEXT, CHROMA_TEMP_LONGTEXT, false)
+            CHROMA_TEMP_TEXT, CHROMA_TEMP_TEXT, false)
 
     add_shortcut("hqdn3d")
 
@@ -92,12 +91,10 @@ struct filter_sys_t
     const vlc_chroma_description_t *chroma;
     int w[3], h[3];
 
-    float luma_spat;
-    float chroma_spat;
-    float luma_temp;
-    float chroma_temp;
-
     struct vf_priv_s cfg;
+    bool   b_recalc_coefs;
+    vlc_mutex_t coefs_mutex;
+    float  luma_spat, luma_temp, chroma_spat, chroma_temp;
 };
 
 /*****************************************************************************
@@ -140,31 +137,30 @@ static int Open(vlc_object_t *this)
         if (sys->w[i] > wmax) wmax = sys->w[i];
         sys->h[i] = fmt_out->i_height * chroma->p[i].h.num / chroma->p[i].h.den;
     }
-    cfg->Line = malloc(wmax*sizeof(int));
+    cfg->Line = malloc(wmax*sizeof(unsigned int));
     if (!cfg->Line) {
         free(sys);
         return VLC_ENOMEM;
     }
 
-    filter->p_sys = sys;
-    filter->pf_video_filter = Filter;
-
     config_ChainParse(filter, FILTER_PREFIX, filter_options,
                       filter->p_cfg);
 
-    sys->luma_spat =
-            var_CreateGetFloatCommand(filter, FILTER_PREFIX "luma-spat");
-    sys->chroma_spat =
-            var_CreateGetFloatCommand(filter, FILTER_PREFIX "chroma-spat");
-    sys->luma_temp =
-            var_CreateGetFloatCommand(filter, FILTER_PREFIX "luma-temp");
-    sys->chroma_temp =
-            var_CreateGetFloatCommand(filter, FILTER_PREFIX "chroma-temp");
 
-    PrecalcCoefs(cfg->Coefs[0], sys->luma_spat);
-    PrecalcCoefs(cfg->Coefs[1], sys->luma_temp);
-    PrecalcCoefs(cfg->Coefs[2], sys->chroma_spat);
-    PrecalcCoefs(cfg->Coefs[3], sys->chroma_temp);
+    vlc_mutex_init( &sys->coefs_mutex );
+    sys->b_recalc_coefs = true;
+    sys->luma_spat = var_CreateGetFloatCommand(filter, FILTER_PREFIX "luma-spat");
+    sys->chroma_spat = var_CreateGetFloatCommand(filter, FILTER_PREFIX "chroma-spat");
+    sys->luma_temp = var_CreateGetFloatCommand(filter, FILTER_PREFIX "luma-temp");
+    sys->chroma_temp = var_CreateGetFloatCommand(filter, FILTER_PREFIX "chroma-temp");
+
+    filter->p_sys = sys;
+    filter->pf_video_filter = Filter;
+
+    var_AddCallback( filter, FILTER_PREFIX "luma-spat", DenoiseCallback, sys );
+    var_AddCallback( filter, FILTER_PREFIX "chroma-spat", DenoiseCallback, sys );
+    var_AddCallback( filter, FILTER_PREFIX "luma-temp", DenoiseCallback, sys );
+    var_AddCallback( filter, FILTER_PREFIX "chroma-temp", DenoiseCallback, sys );
 
     return VLC_SUCCESS;
 }
@@ -178,6 +174,13 @@ static void Close(vlc_object_t *this)
     filter_sys_t *sys = filter->p_sys;
     struct vf_priv_s *cfg = &sys->cfg;
 
+    var_DelCallback( filter, FILTER_PREFIX "luma-spat", DenoiseCallback, sys );
+    var_DelCallback( filter, FILTER_PREFIX "chroma-spat", DenoiseCallback, sys );
+    var_DelCallback( filter, FILTER_PREFIX "luma-temp", DenoiseCallback, sys );
+    var_DelCallback( filter, FILTER_PREFIX "chroma-temp", DenoiseCallback, sys );
+
+    vlc_mutex_destroy( &sys->coefs_mutex );
+
     for (int i = 0; i < 3; ++i) {
         free(cfg->Frame[i]);
     }
@@ -193,6 +196,7 @@ static picture_t *Filter(filter_t *filter, picture_t *src)
     picture_t *dst;
     filter_sys_t *sys = filter->p_sys;
     struct vf_priv_s *cfg = &sys->cfg;
+    bool recalc = false;
 
     if (!src) return NULL;
 
@@ -201,6 +205,20 @@ static picture_t *Filter(filter_t *filter, picture_t *src)
         picture_Release(src);
         return NULL;
     }
+    vlc_mutex_lock( &sys->coefs_mutex );
+    recalc = sys->b_recalc_coefs;
+    sys->b_recalc_coefs = false;
+
+    if( unlikely( recalc ) )
+    {
+        msg_Dbg( filter, "Changing coefs to %.2f %.2f %.2f %.2f",
+                            sys->luma_spat, sys->luma_temp, sys->chroma_spat, sys->chroma_temp );
+        PrecalcCoefs(cfg->Coefs[0], sys->luma_spat);
+        PrecalcCoefs(cfg->Coefs[1], sys->luma_temp);
+        PrecalcCoefs(cfg->Coefs[2], sys->chroma_spat);
+        PrecalcCoefs(cfg->Coefs[3], sys->chroma_temp);
+    }
+    vlc_mutex_unlock( &sys->coefs_mutex );
 
     deNoise(src->p[0].p_pixels, dst->p[0].p_pixels,
             cfg->Line, &cfg->Frame[0], sys->w[0], sys->h[0],
@@ -223,3 +241,29 @@ static picture_t *Filter(filter_t *filter, picture_t *src)
 
     return CopyInfoAndRelease(dst, src);
 }
+
+
+static int DenoiseCallback( vlc_object_t *p_this, char const *psz_var,
+                            vlc_value_t oldval, vlc_value_t newval,
+                            void *p_data )
+{
+    VLC_UNUSED(p_this); VLC_UNUSED(oldval);
+
+    filter_sys_t *sys = (filter_sys_t*)p_data;
+
+    /* Just take values and flag for recalc so we don't block UI thread calling this
+     * and don't right thread safety calcing coefs in here without mutex*/
+    vlc_mutex_lock( &sys->coefs_mutex );
+    if( !strcmp( psz_var, FILTER_PREFIX "luma-spat") )
+        sys->luma_spat = newval.f_float;
+    else if( !strcmp( psz_var, FILTER_PREFIX "luma-temp") )
+        sys->luma_temp = newval.f_float;
+    else if( !strcmp( psz_var, FILTER_PREFIX "chroma-temp") )
+        sys->chroma_spat = newval.f_float;
+    else if( !strcmp( psz_var, FILTER_PREFIX "chroma-spat") )
+        sys->chroma_temp = newval.f_float;
+    sys->b_recalc_coefs = true;
+    vlc_mutex_unlock( &sys->coefs_mutex );
+
+    return VLC_SUCCESS;
+}
