@@ -1,25 +1,25 @@
 /*****************************************************************************
  * effects.c : Effects for the visualization system
  *****************************************************************************
- * Copyright (C) 2002-2009 the VideoLAN team
- * $Id: 110337523c794bad070b7c11b409247efaecd079 $
+ * Copyright (C) 2002-2009 VLC authors and VideoLAN
+ * $Id: 7b09890f32f5d08c5487bafc85bc760dbcf0ebf2 $
  *
  * Authors: Clément Stenac <zorglub@via.ecp.fr>
  *          Adrien Maglo <magsoft@videolan.org>
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU Lesser General Public License as published by
+ * the Free Software Foundation; either version 2.1 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
+ * You should have received a copy of the GNU Lesser General Public License
+ * along with this program; if not, write to the Free Software Foundation,
+ * Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
  *****************************************************************************/
 
 /*****************************************************************************
@@ -37,6 +37,7 @@
 #include <math.h>
 
 #include "fft.h"
+#include "window.h"
 
 #define PEAK_SPEED 1
 #define BAR_DECREASE_SPEED 5
@@ -48,19 +49,36 @@
 /*****************************************************************************
  * dummy_Run
  *****************************************************************************/
-int dummy_Run( visual_effect_t * p_effect, vlc_object_t *p_aout,
-               const block_t * p_buffer , picture_t * p_picture)
+static int dummy_Run( visual_effect_t * p_effect, vlc_object_t *p_aout,
+                      const block_t * p_buffer , picture_t * p_picture)
 {
     VLC_UNUSED(p_effect); VLC_UNUSED(p_aout); VLC_UNUSED(p_buffer);
     VLC_UNUSED(p_picture);
     return 0;
 }
 
+static void dummy_Free( void *data )
+{
+    VLC_UNUSED(data);
+}
+
+
 /*****************************************************************************
  * spectrum_Run: spectrum analyser
  *****************************************************************************/
-int spectrum_Run(visual_effect_t * p_effect, vlc_object_t *p_aout,
-                 const block_t * p_buffer , picture_t * p_picture)
+typedef struct spectrum_data
+{
+    int *peaks;
+    int *prev_heights;
+
+    unsigned i_prev_nb_samples;
+    int16_t *p_prev_s16_buff;
+
+    window_param wind_param;
+} spectrum_data;
+
+static int spectrum_Run(visual_effect_t * p_effect, vlc_object_t *p_aout,
+                        const block_t * p_buffer , picture_t * p_picture)
 {
     spectrum_data *p_data = p_effect->p_data;
     float p_output[FFT_BUFFER_SIZE];  /* Raw FFT Result  */
@@ -87,6 +105,7 @@ int spectrum_Run(visual_effect_t * p_effect, vlc_object_t *p_aout,
     const int *xscale;
 
     fft_state *p_state;                 /* internal FFT data */
+    DEFINE_WIND_CONTEXT( wind_ctx );    /* internal window data */
 
     int i , j , y , k;
     int i_line;
@@ -100,6 +119,11 @@ int spectrum_Run(visual_effect_t * p_effect, vlc_object_t *p_aout,
     int16_t  *p_buffs;                    /* int16_t converted buffer */
     int16_t  *p_s16_buff;                 /* int16_t converted buffer */
 
+    if (!p_buffer->i_nb_samples) {
+        msg_Err(p_aout, "no samples yet");
+        return -1;
+    }
+
     /* Create p_data if needed */
     if( !p_data )
     {
@@ -112,6 +136,8 @@ int spectrum_Run(visual_effect_t * p_effect, vlc_object_t *p_aout,
 
         p_data->i_prev_nb_samples = 0;
         p_data->p_prev_s16_buff = NULL;
+
+        window_get_param( p_aout, &p_data->wind_param );
     }
     peaks = (int *)p_data->peaks;
     prev_heights = (int *)p_data->prev_heights;
@@ -167,6 +193,13 @@ int spectrum_Run(visual_effect_t * p_effect, vlc_object_t *p_aout,
         msg_Err(p_aout,"unable to initialize FFT transform");
         return -1;
     }
+    if( !window_init( FFT_BUFFER_SIZE, &p_data->wind_param, &wind_ctx ) )
+    {
+        fft_close( p_state );
+        free( height );
+        msg_Err(p_aout,"unable to initialize FFT window");
+        return -1;
+    }
     p_buffs = p_s16_buff;
     for ( i = 0 ; i < FFT_BUFFER_SIZE ; i++)
     {
@@ -178,6 +211,7 @@ int spectrum_Run(visual_effect_t * p_effect, vlc_object_t *p_aout,
             p_buffs = p_s16_buff;
 
     }
+    window_scale_in_place( p_buffer1, &wind_ctx );
     fft_perform( p_buffer1, p_output, p_state);
     for( i = 0; i< FFT_BUFFER_SIZE ; i++ )
         p_dest[i] = p_output[i] *  ( 2 ^ 16 ) / ( ( FFT_BUFFER_SIZE / 2 * 32768 ) ^ 2 );
@@ -325,6 +359,8 @@ int spectrum_Run(visual_effect_t * p_effect, vlc_object_t *p_aout,
         }
     }
 
+    window_close( &wind_ctx );
+
     fft_close( p_state );
 
     free( height );
@@ -332,12 +368,35 @@ int spectrum_Run(visual_effect_t * p_effect, vlc_object_t *p_aout,
     return 0;
 }
 
+static void spectrum_Free( void *data )
+{
+    spectrum_data *p_data = data;
+
+    if( p_data != NULL )
+    {
+        free( p_data->peaks );
+        free( p_data->prev_heights );
+        free( p_data->p_prev_s16_buff );
+        free( p_data );
+    }
+}
+
 
 /*****************************************************************************
  * spectrometer_Run: derivative spectrum analysis
  *****************************************************************************/
-int spectrometer_Run(visual_effect_t * p_effect, vlc_object_t *p_aout,
-                     const block_t * p_buffer , picture_t * p_picture)
+typedef struct
+{
+    int *peaks;
+
+    unsigned i_prev_nb_samples;
+    int16_t *p_prev_s16_buff;
+
+    window_param wind_param;
+} spectrometer_data;
+
+static int spectrometer_Run(visual_effect_t * p_effect, vlc_object_t *p_aout,
+                            const block_t * p_buffer , picture_t * p_picture)
 {
 #define Y(R,G,B) ((uint8_t)( (R * .299) + (G * .587) + (B * .114) ))
 #define U(R,G,B) ((uint8_t)( (R * -.169) + (G * -.332) + (B * .500) + 128 ))
@@ -384,6 +443,7 @@ int spectrometer_Run(visual_effect_t * p_effect, vlc_object_t *p_aout,
     const double y_scale =  3.60673760222;  /* (log 256) */
 
     fft_state *p_state;                 /* internal FFT data */
+    DEFINE_WIND_CONTEXT( wind_ctx );    /* internal window data */
 
     int i , j , k;
     int i_line = 0;
@@ -396,6 +456,11 @@ int spectrometer_Run(visual_effect_t * p_effect, vlc_object_t *p_aout,
     int16_t  *p_buffs;                    /* int16_t converted buffer */
     int16_t  *p_s16_buff;                /* int16_t converted buffer */
 
+    if (!p_buffer->i_nb_samples) {
+        msg_Err(p_aout, "no samples yet");
+        return -1;
+    }
+
     /* Create the data struct if needed */
     spectrometer_data *p_data = p_effect->p_data;
     if( !p_data )
@@ -411,6 +476,7 @@ int spectrometer_Run(visual_effect_t * p_effect, vlc_object_t *p_aout,
         }
         p_data->i_prev_nb_samples = 0;
         p_data->p_prev_s16_buff = NULL;
+        window_get_param( p_aout, &p_data->wind_param );
         p_effect->p_data = (void*)p_data;
     }
     peaks = p_data->peaks;
@@ -475,6 +541,13 @@ int spectrometer_Run(visual_effect_t * p_effect, vlc_object_t *p_aout,
         free( height );
         return -1;
     }
+    if( !window_init( FFT_BUFFER_SIZE, &p_data->wind_param, &wind_ctx ) )
+    {
+        fft_close( p_state );
+        free( height );
+        msg_Err(p_aout,"unable to initialize FFT window");
+        return -1;
+    }
     p_buffs = p_s16_buff;
     for ( i = 0 ; i < FFT_BUFFER_SIZE; i++)
     {
@@ -485,6 +558,7 @@ int spectrometer_Run(visual_effect_t * p_effect, vlc_object_t *p_aout,
         if( p_buffs >= &p_s16_buff[p_buffer->i_nb_samples * p_effect->i_nb_chans] )
             p_buffs = p_s16_buff;
     }
+    window_scale_in_place( p_buffer1, &wind_ctx );
     fft_perform( p_buffer1, p_output, p_state);
     for(i = 0; i < FFT_BUFFER_SIZE; i++)
     {
@@ -782,6 +856,8 @@ int spectrometer_Run(visual_effect_t * p_effect, vlc_object_t *p_aout,
         }
     }
 
+    window_close( &wind_ctx );
+
     fft_close( p_state );
 
     free( height );
@@ -789,12 +865,24 @@ int spectrometer_Run(visual_effect_t * p_effect, vlc_object_t *p_aout,
     return 0;
 }
 
+static void spectrometer_Free( void *data )
+{
+    spectrometer_data *p_data = data;
+
+    if( p_data != NULL )
+    {
+        free( p_data->peaks );
+        free( p_data->p_prev_s16_buff );
+        free( p_data );
+    }
+}
+
 
 /*****************************************************************************
  * scope_Run: scope effect
  *****************************************************************************/
-int scope_Run(visual_effect_t * p_effect, vlc_object_t *p_aout,
-              const block_t * p_buffer , picture_t * p_picture)
+static int scope_Run(visual_effect_t * p_effect, vlc_object_t *p_aout,
+                     const block_t * p_buffer , picture_t * p_picture)
 {
     VLC_UNUSED(p_aout);
 
@@ -807,8 +895,8 @@ int scope_Run(visual_effect_t * p_effect, vlc_object_t *p_aout,
         for( int j = 0 ; j < 3 ; j++ )
         {
             ppp_area[i_index][j] =
-                p_picture->p[j].p_pixels + i_index * p_picture->p[j].i_lines
-                / 2 * p_picture->p[j].i_pitch;
+                p_picture->p[j].p_pixels + (i_index * 2 + 1) * p_picture->p[j].i_lines
+                / 4 * p_picture->p[j].i_pitch;
         }
     }
 
@@ -816,10 +904,10 @@ int scope_Run(visual_effect_t * p_effect, vlc_object_t *p_aout,
             i_index < __MIN( p_effect->i_width, (int)p_buffer->i_nb_samples );
             i_index++ )
     {
-        uint8_t i_value;
+        int8_t i_value;
 
         /* Left channel */
-        i_value =  (1.0+p_sample[p_effect->i_idx_left]) * 127;
+        i_value =  p_sample[p_effect->i_idx_left] * 127;
         *(ppp_area[0][0]
                 + p_picture->p[0].i_pitch * i_index / p_effect->i_width
                 + p_picture->p[0].i_lines * i_value / 512
@@ -831,7 +919,7 @@ int scope_Run(visual_effect_t * p_effect, vlc_object_t *p_aout,
 
 
         /* Right channel */
-        i_value = (1.0+p_sample[p_effect->i_idx_right]) * 127;
+        i_value = p_sample[p_effect->i_idx_right] * 127;
         *(ppp_area[1][0]
                 + p_picture->p[0].i_pitch * i_index / p_effect->i_width
                 + p_picture->p[0].i_lines * i_value / 512
@@ -850,14 +938,14 @@ int scope_Run(visual_effect_t * p_effect, vlc_object_t *p_aout,
 /*****************************************************************************
  * vuMeter_Run: vu meter effect
  *****************************************************************************/
-int vuMeter_Run(visual_effect_t * p_effect, vlc_object_t *p_aout,
-                const block_t * p_buffer , picture_t * p_picture)
+static int vuMeter_Run(visual_effect_t * p_effect, vlc_object_t *p_aout,
+                       const block_t * p_buffer , picture_t * p_picture)
 {
     VLC_UNUSED(p_aout);
     float i_value_l = 0;
     float i_value_r = 0;
 
-    /* Compute the peack values */
+    /* Compute the peak values */
     for ( unsigned i = 0 ; i < p_buffer->i_nb_samples; i++ )
     {
         const float *p_sample = (float *)p_buffer->p_buffer;
@@ -874,8 +962,8 @@ int vuMeter_Run(visual_effect_t * p_effect, vlc_object_t *p_aout,
         p_sample += p_effect->i_nb_chans;
     }
 
-    i_value_l = abs(i_value_l);
-    i_value_r = abs(i_value_r);
+    i_value_l = fabsf(i_value_l);
+    i_value_r = fabsf(i_value_r);
 
     /* Stay under maximum value admited */
     if ( i_value_l > 200 * M_PI_2 )
@@ -985,3 +1073,13 @@ int vuMeter_Run(visual_effect_t * p_effect, vlc_object_t *p_aout,
 
     return 0;
 }
+
+/* Table of effects */
+const struct visual_cb_t effectv[] = {
+    { "scope",        scope_Run,        dummy_Free        },
+    { "vuMeter",      vuMeter_Run,      dummy_Free        },
+    { "spectrum",     spectrum_Run,     spectrum_Free     },
+    { "spectrometer", spectrometer_Run, spectrometer_Free },
+    { "dummy",        dummy_Run,        dummy_Free        },
+};
+const unsigned effectc = sizeof (effectv) / sizeof (effectv[0]);
