@@ -343,7 +343,7 @@ function help(name,client,arg)
                 end
                 if val.args then str = str .. " " .. val.args end
                 if #str%2 == 1 then str = str .. " " end
-                str = str .. string.rep(" .",(width-(#str+#val.help)-1)/2)
+                str = str .. string.rep(" .",math.floor((width-(#str+#val.help)-1)/2))
                 str = str .. string.rep(" ",width-#str-#val.help) .. val.help
             end
             client:append(str)
