@@ -21,18 +21,6 @@
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
 --]]
 
-function get_prefres()
-    local prefres = -1
-    if vlc.var and vlc.var.inherit then
-        prefres = vlc.var.inherit(nil, "preferred-resolution")
-        if prefres == nil then
-            prefres = -1
-        end
-    end
-
-    return prefres
-end
-
 -- Probe function.
 function probe()
 	if vlc.access ~= "http" and vlc.access ~= "https" then
@@ -44,7 +32,7 @@ end
 
 -- Parse function.
 function parse()
-	prefres = get_prefres()
+	prefres = vlc.var.inherit(nil, "preferred-resolution")
 
 
 	while true
