@@ -1,7 +1,7 @@
 --[[
  $Id$
 
- Copyright © 2007-2013 the VideoLAN team
+ Copyright © 2007-2015 the VideoLAN team
 
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
@@ -33,23 +33,12 @@ function get_arturl()
     if not video_id then
         return nil
     end
-    return "http://img.youtube.com/vi/"..video_id.."/default.jpg"
-end
-
-function get_prefres()
-    local prefres = -1
-    if vlc.var and vlc.var.inherit then
-        prefres = vlc.var.inherit(nil, "preferred-resolution")
-        if prefres == nil then
-            prefres = -1
-        end
-    end
-    return prefres
+    return vlc.access.."://img.youtube.com/vi/"..video_id.."/default.jpg"
 end
 
 -- Pick the most suited format available
 function get_fmt( fmt_list )
-    local prefres = get_prefres()
+    local prefres = vlc.var.inherit(nil, "preferred-resolution")
     if prefres < 0 then
         return nil
     end
@@ -67,112 +56,128 @@ function get_fmt( fmt_list )
     return fmt
 end
 
+-- Buffering iterator to parse through the HTTP stream several times
+-- without making several HTTP requests
+function buf_iter( s )
+    s.i = s.i + 1
+    local line = s.lines[s.i]
+    if not line then
+        -- Put back together statements split across several lines,
+        -- otherwise we won't be able to parse them
+        repeat
+            local l = s.stream:readline()
+            if not l then break end
+            line = line and line..l or l -- Ternary operator
+        until string.match( line, "};$" )
+
+        if line then
+            s.lines[s.i] = line
+        end
+    end
+    return line
+end
+
+-- Helper to search and extract code from javascript stream
+function js_extract( js, pattern, alt )
+    js.i = 0 -- Reset to beginning
+    for line in buf_iter, js do
+        local ex = string.match( line, pattern )
+        if not ex and alt then
+            ex = string.match( line, alt )
+        end
+        if ex then
+            return ex
+        end
+    end
+    vlc.msg.err( "Couldn't process youtube video URL, please check for updates to this script" )
+    return nil
+end
+
 -- Descramble the URL signature using the javascript code that does that
 -- in the web page
 function js_descramble( sig, js_url )
     -- Fetch javascript code
-    local js = vlc.stream( js_url )
-    if not js then
+    local js = { stream = vlc.stream( js_url ), lines = {}, i = 0 }
+    if not js.stream then
+        vlc.msg.err( "Couldn't process youtube video URL, please check for updates to this script" )
         return sig
     end
-    local lines = {}
 
     -- Look for the descrambler function's name
-    local descrambler = nil
-    while not descrambler do
-        local line = js:readline()
-        if not line then
-            vlc.msg.err( "Couldn't process youtube video URL, please check for updates to this script" )
-            return sig
-        end
-        -- Buffer lines for later, so we don't have to make a second
-        -- HTTP request later
-        table.insert( lines, line )
-        -- c&&(b.signature=ij(c));
-        descrambler = string.match( line, "%.signature=(.-)%(" )
+    -- c&&a.set("signature",br(c));
+    local descrambler = js_extract( js, "%.set%(\"signature\",(.-)%(", nil )
+    if not descrambler then
+        return sig
     end
 
-    -- Fetch the code of the descrambler function. Example:
-    -- function ij(a){a=a.split("");a=a.reverse();a=jj(a,12);a=jj(a,32);a=a.reverse();a=jj(a,34);a=a.slice(3);a=jj(a,35);a=jj(a,42);a=a.slice(2);return a.join("")}
-    local rules = nil
-    while not rules do
-        local line
-        if #lines > 0 then
-            line = table.remove( lines )
+    -- Fetch the code of the descrambler function
+    -- var Go=function(a){a=a.split("");Fo.sH(a,2);Fo.TU(a,28);Fo.TU(a,44);Fo.TU(a,26);Fo.TU(a,40);Fo.TU(a,64);Fo.TR(a,26);Fo.sH(a,1);return a.join("")};
+    local rules = js_extract( js, "[ ,]"..descrambler.."=function%([^)]*%){(.-)};",
+                                  -- Legacy/alternate format
+                                  "function "..descrambler.."%([^)]*%){(.-)}" )
+    if not rules then
+        return sig
+    end
+
+    -- Get the name of the helper object providing transformation definitions
+    local helper = string.match( rules, ";(..)%...%(" )
+    if not helper then
+        vlc.msg.err( "Couldn't process youtube video URL, please check for updates to this script" )
+        return sig
+    end
+
+    -- Fetch the helper object code
+    -- var Fo={TR:function(a){a.reverse()},TU:function(a,b){var c=a[0];a[0]=a[b%a.length];a[b]=c},sH:function(a,b){a.splice(0,b)}};
+    local transformations = js_extract( js, "[ ,]"..helper.."={(.-)};", nil )
+    if not transformations then
+        return sig
+    end
+
+    -- Parse the helper object to map available transformations
+    local trans = {}
+    for meth,code in string.gmatch( transformations, "(..):function%([^)]*%){([^}]*)}" ) do
+        -- a=a.reverse()
+        if string.match( code, "%.reverse%(" ) then
+          trans[meth] = "reverse"
+
+        -- a.splice(0,b)
+        elseif string.match( code, "%.splice%(") then
+          trans[meth] = "slice"
+
+        -- var c=a[0];a[0]=a[b%a.length];a[b]=c
+        elseif string.match( code, "var c=" ) then
+          trans[meth] = "swap"
         else
-            line = js:readline()
-            if not line then
-                vlc.msg.err( "Couldn't process youtube video URL, please check for updates to this script" )
-                return sig
-            end
+            vlc.msg.warn("Couldn't parse unknown youtube video URL signature transformation")
         end
-        rules = string.match( line, "function "..descrambler.."%([^)]*%){(.-)}" )
     end
 
-    -- Parse descrambling rules one by one and apply them on the
-    -- signature as we go
-    for rule in string.gmatch( rules, "[^;]+" ) do
-        -- a=a.reverse();
-        if string.match( rule, "%.reverse%(" ) then
+    -- Parse descrambling rules, map them to known transformations
+    -- and apply them on the signature
+    local missing = false
+    for meth,idx in string.gmatch( rules, "..%.(..)%([^,]+,(%d+)%)" ) do
+        idx = tonumber( idx )
+
+        if trans[meth] == "reverse" then
             sig = string.reverse( sig )
-        else
 
-        -- a=a.slice(3);
-        local len = string.match( rule, "%.slice%((%d+)%)" )
-        if len then
-            sig = string.sub( sig, len + 1 )
-        else
+        elseif trans[meth] == "slice" then
+            sig = string.sub( sig, idx + 1 )
 
-        -- a=jj(a,32);
-        -- This is known to be a function swapping the first and nth
-        -- characters:
-        -- function jj(a,b){var c=a[0];a[0]=a[b%a.length];a[b]=c;return a}
-        local idx = string.match( rule, "=..%([^,]+,(%d+)%)" )
-        -- This swapping function may also appear inlined:
-        -- var b=a[0];a[0]=a[59%a.length];a[59]=b;
-        -- In that case we only catch one of the three rules.
-        if not idx then
-            idx = string.match( rule, ".%[(%d+)%]=." )
-        end
-        if idx then
-            idx = tonumber( idx )
-            if not idx then idx = 0 end
+        elseif trans[meth] == "swap" then
             if idx > 1 then
                 sig = string.gsub( sig, "^(.)("..string.rep( ".", idx - 1 )..")(.)(.*)$", "%3%2%1%4" )
             elseif idx == 1 then
                 sig = string.gsub( sig, "^(.)(.)", "%2%1" )
             end
-        end end end
-
-        -- Simply ignore other statements, in particular initial split
-        -- and final join and return statements
-    end
-    return sig
-end
-
-function descramble81( sig )
-    sig = string.reverse( sig )
-    local s1,s2,s3,s4,s5,s6,s7,s8,s9,s10,s11,s12,s13 =
-        string.match( sig, "(.)(.......................)(.)(..............)(.)(......)(.)(....)(.)(...................)(.)(........)(.)" )
-    return s3..s2..s5..s4..s1..s6..s13..s8..s7..s10..s9..s12..s11
-end
-
-local descramblers = {
-                       --[81] = descramble81
-                     }
-
-function descramble( sig, js_url )
-    vlc.msg.dbg( "Found "..string.len( sig ).."-character scrambled signature for youtube video URL, attempting to descramble... " )
-    if js_url then
-        sig = js_descramble( sig, js_url )
-    else
-        local descrambler = descramblers[string.len( sig )]
-        if descrambler then
-            sig = descrambler( sig )
         else
-            vlc.msg.err( "Couldn't process youtube video URL, please check for updates to this script" )
+            vlc.msg.dbg("Couldn't apply unknown youtube video URL signature transformation")
+            missing = true
         end
     end
+    if missing then
+        vlc.msg.err( "Couldn't process youtube video URL, please check for updates to this script" )
+    end
     return sig
 end
 
@@ -193,7 +198,12 @@ function pick_url( url_map, fmt, js_url )
                     -- Scrambled signature
                     sig = string.match( stream, "s=([^&,]+)" )
                     if sig then
-                        sig = descramble( sig, js_url )
+                        vlc.msg.dbg( "Found "..string.len( sig ).."-character scrambled signature for youtube video URL, attempting to descramble... " )
+                        if js_url then
+                            sig = js_descramble( sig, js_url )
+                        else
+                            vlc.msg.err( "Couldn't process youtube video URL, please check for updates to this script" )
+                        end
                     end
                 end
                 local signature = ""
@@ -226,6 +236,7 @@ function probe()
     return (  string.match( vlc.path, "/watch%?" ) -- the html page
             or string.match( vlc.path, "/get_video_info%?" ) -- info API
             or string.match( vlc.path, "/v/" ) -- video in swf player
+            or string.match( vlc.path, "/embed/" ) -- embedded player iframe
             or string.match( vlc.path, "/player2.swf" ) ) -- another player url
 end
 
@@ -240,47 +251,46 @@ function parse()
             -- Try to find the video's title
             line = vlc.readline()
             if not line then break end
-            if string.match( line, "<meta name=\"title\"" ) then
+            if string.match( line, "<meta property=\"og:title\"" ) then
                 _,_,name = string.find( line, "content=\"(.-)\"" )
                 name = vlc.strings.resolve_xml_special_chars( name )
                 name = vlc.strings.resolve_xml_special_chars( name )
             end
-            if string.match( line, "<meta name=\"description\"" ) then
-               -- Don't ask me why they double encode ...
-                _,_,description = string.find( line, "content=\"(.-)\"" )
-                description = vlc.strings.resolve_xml_special_chars( description )
+
+            if string.match( line, "<p id=\"eow[-]description\" >" ) then
+                _,_,description = string.find( line, "<p id=\"eow[-]description\" >(.-)<[/]p>" )
                 description = vlc.strings.resolve_xml_special_chars( description )
             end
+
+
             if string.match( line, "<meta property=\"og:image\"" ) then
                 _,_,arturl = string.find( line, "content=\"(.-)\"" )
+                arturl = vlc.strings.resolve_xml_special_chars( arturl )
             end
-            -- This is not available in the video parameters (whereas it
-            -- is given by the get_video_info API as the "author" field)
-            if not artist then
-                artist = string.match( line, "yt%-uix%-sessionlink yt%-user%-name[^>]*>([^<]*)</" )
-                if artist then
-                    artist = vlc.strings.resolve_xml_special_chars( artist )
-                end
+
+            if string.match(line, "\"author\": *\"(.-)\"")    then
+                _,_,artist = string.find(line, "\"author\": *\"(.-)\"")
             end
+
             -- JSON parameters, also formerly known as "swfConfig",
             -- "SWF_ARGS", "swfArgs", "PLAYER_CONFIG", "playerConfig" ...
             if string.match( line, "ytplayer%.config" ) then
 
-                local js_url = string.match( line, "\"js\": \"(.-)\"" )
+                local js_url = string.match( line, "\"js\": *\"(.-)\"" )
                 if js_url then
                     js_url = string.gsub( js_url, "\\/", "/" )
                     js_url = string.gsub( js_url, "^//", vlc.access.."://" )
                 end
 
                 if not fmt then
-                    fmt_list = string.match( line, "\"fmt_list\": \"(.-)\"" )
+                    fmt_list = string.match( line, "\"fmt_list\": *\"(.-)\"" )
                     if fmt_list then
                         fmt_list = string.gsub( fmt_list, "\\/", "/" )
                         fmt = get_fmt( fmt_list )
                     end
                 end
 
-                url_map = string.match( line, "\"url_encoded_fmt_stream_map\": \"(.-)\"" )
+                url_map = string.match( line, "\"url_encoded_fmt_stream_map\": *\"(.-)\"" )
                 if url_map then
                     -- FIXME: do this properly
                     url_map = string.gsub( url_map, "\\u0026", "&" )
@@ -289,8 +299,8 @@ function parse()
 
                 if not path then
                     -- If this is a live stream, the URL map will be empty
-                    -- and we get the URL from this field instead 
-                    local hlsvp = string.match( line, "\"hlsvp\": \"(.-)\"" )
+                    -- and we get the URL from this field instead
+                    local hlsvp = string.match( line, "\"hlsvp\": *\"(.-)\"" )
                     if hlsvp then
                         hlsvp = string.gsub( hlsvp, "\\/", "/" )
                         path = hlsvp
@@ -309,11 +319,11 @@ function parse()
                     format = "&fmt=" .. fmt
                 else
                     format = ""
-                end 
+                end
                 -- Without "el=detailpage", /get_video_info fails for many
                 -- music videos with errors about copyrighted content being
                 -- "restricted from playback on certain sites"
-                path = "http://www.youtube.com/get_video_info?video_id="..video_id..format.."&el=detailpage"
+                path = vlc.access.."://www.youtube.com/get_video_info?video_id="..video_id..format.."&el=detailpage"
                 vlc.msg.warn( "Couldn't extract video URL, falling back to alternate youtube API" )
             end
         end
@@ -349,7 +359,7 @@ function parse()
 
         if not path then
             -- If this is a live stream, the URL map will be empty
-            -- and we get the URL from this field instead 
+            -- and we get the URL from this field instead
             local hlsvp = string.match( line, "&hlsvp=([^&]*)" )
             if hlsvp then
                 hlsvp = vlc.strings.decode_uri( hlsvp )
@@ -384,6 +394,9 @@ function parse()
         if not video_id then
             _,_,video_id = string.find( vlc.path, "/v/([^?]*)" )
         end
+        if not video_id then
+            video_id = string.match( vlc.path, "/embed/([^?]*)" )
+        end
         if not video_id then
             vlc.msg.err( "Couldn't extract youtube video URL" )
             return { }
@@ -394,6 +407,6 @@ function parse()
         else
             format = ""
         end
-        return { { path = "http://www.youtube.com/watch?v="..video_id..format } }
+        return { { path = vlc.access.."://www.youtube.com/watch?v="..video_id..format } }
     end
 end
