@@ -2,7 +2,7 @@
  * dec.c : audio output API towards decoders
  *****************************************************************************
  * Copyright (C) 2002-2007 VLC authors and VideoLAN
- * $Id: 57d292fe0a9123beb2c557fd98e19816c3980805 $
+ * $Id: ef03894b99e5f8ce6bfa5330165e8c837ab0a5e9 $
  *
  * Authors: Christophe Massiot <massiot@via.ecp.fr>
  *
@@ -92,6 +92,7 @@ int aout_DecNew( audio_output_t *p_aout,
         aout_OutputDelete (p_aout);
 error:
         aout_volume_Delete (owner->volume);
+        owner->volume = NULL;
         aout_OutputUnlock (p_aout);
         return -1;
     }
@@ -119,6 +120,7 @@ void aout_DecDelete (audio_output_t *aout)
         aout_OutputDelete (aout);
     }
     aout_volume_Delete (owner->volume);
+    owner->volume = NULL;
     aout_OutputUnlock (aout);
     var_Destroy (aout, "stereo-mode");
 }
@@ -182,34 +184,6 @@ void aout_RequestRestart (audio_output_t *aout, unsigned mode)
  * Buffer management
  */
 
-/*****************************************************************************
- * aout_DecNewBuffer : ask for a new empty buffer
- *****************************************************************************/
-block_t *aout_DecNewBuffer (audio_output_t *aout, size_t samples)
-{
-    /* NOTE: the caller is responsible for serializing input change */
-    aout_owner_t *owner = aout_owner (aout);
-
-    size_t length = samples * owner->input_format.i_bytes_per_frame
-                            / owner->input_format.i_frame_length;
-    block_t *block = block_Alloc( length );
-    if( likely(block != NULL) )
-    {
-        block->i_nb_samples = samples;
-        block->i_pts = block->i_length = 0;
-    }
-    return block;
-}
-
-/*****************************************************************************
- * aout_DecDeleteBuffer : destroy an undecoded buffer
- *****************************************************************************/
-void aout_DecDeleteBuffer (audio_output_t *aout, block_t *block)
-{
-    (void) aout;
-    block_Release (block);
-}
-
 static void aout_StopResampling (audio_output_t *aout)
 {
     aout_owner_t *owner = aout_owner (aout);
