@@ -2,7 +2,7 @@
  * output.c : internal management of output streams for the audio output
  *****************************************************************************
  * Copyright (C) 2002-2004 VLC authors and VideoLAN
- * $Id: 988d200be72f8c2eaeb9a05e5e4546aa1aaa8a42 $
+ * $Id: ad9f6760b436f1c8341a931db61aeb23b878cd71 $
  *
  * Authors: Christophe Massiot <massiot@via.ecp.fr>
  *
@@ -153,6 +153,15 @@ static int aout_GainNotify (audio_output_t *aout, float gain)
     return 0;
 }
 
+static int FilterCallback (vlc_object_t *obj, const char *var,
+                           vlc_value_t prev, vlc_value_t cur, void *data)
+{
+    if (strcmp(prev.psz_string, cur.psz_string))
+        aout_InputRequestRestart ((audio_output_t *)obj);
+    (void) var; (void) data;
+    return VLC_SUCCESS;
+}
+
 #undef aout_New
 /**
  * Creates an audio output object and initializes an output module.
@@ -238,6 +247,7 @@ audio_output_t *aout_New (vlc_object_t *parent)
         text.psz_string = (char *)"Goom";
         var_Change (aout, "visual", VLC_VAR_ADDCHOICE, &val, &text);
     }
+#ifndef _WIN32
     /* Look for libprojectM plugin */
     if (module_exists ("projectm"))
     {
@@ -252,6 +262,14 @@ audio_output_t *aout_New (vlc_object_t *parent)
         text.psz_string = (char*)"Vovoid VSXu";
         var_Change (aout, "visual", VLC_VAR_ADDCHOICE, &val, &text);
     }
+    /* Look for glspectrum plugin */
+    if (module_exists ("glspectrum"))
+    {
+        val.psz_string = (char *)"glspectrum";
+        text.psz_string = (char*)"3D spectrum";
+        var_Change (aout, "visual", VLC_VAR_ADDCHOICE, &val, &text);
+    }
+#endif
     str = var_GetNonEmptyString (aout, "effect-list");
     if (str != NULL)
     {
@@ -259,23 +277,8 @@ audio_output_t *aout_New (vlc_object_t *parent)
         free (str);
     }
 
-    /* Equalizer */
-    var_Create (aout, "equalizer", VLC_VAR_STRING | VLC_VAR_HASCHOICE);
-    text.psz_string = _("Equalizer");
-    var_Change (aout, "equalizer", VLC_VAR_SETTEXT, &text, NULL);
-    val.psz_string = (char*)"";
-    text.psz_string = _("Disable");
-    var_Change (aout, "equalizer", VLC_VAR_ADDCHOICE, &val, &text);
-    cfg = config_FindConfig (VLC_OBJECT(aout), "equalizer-preset");
-    if (likely(cfg != NULL))
-        for (unsigned i = 0; i < cfg->list_count; i++)
-        {
-            val.psz_string = cfg->list.psz[i];
-            text.psz_string = vlc_gettext(cfg->list_text[i]);
-            var_Change (aout, "equalizer", VLC_VAR_ADDCHOICE, &val, &text);
-        }
-
     var_Create (aout, "audio-filter", VLC_VAR_STRING | VLC_VAR_DOINHERIT);
+    var_AddCallback (aout, "audio-filter", FilterCallback, NULL);
     text.psz_string = _("Audio filters");
     var_Change (aout, "audio-filter", VLC_VAR_SETTEXT, &text, NULL);
 
@@ -299,6 +302,11 @@ audio_output_t *aout_New (vlc_object_t *parent)
                             &val, &text);
         }
 
+    /* Equalizer */
+    var_Create (aout, "equalizer-preamp", VLC_VAR_FLOAT | VLC_VAR_DOINHERIT);
+    var_Create (aout, "equalizer-bands", VLC_VAR_STRING | VLC_VAR_DOINHERIT);
+    var_Create (aout, "equalizer-preset", VLC_VAR_STRING | VLC_VAR_DOINHERIT);
+
     return aout;
 }
 
@@ -314,8 +322,10 @@ void aout_Destroy (audio_output_t *aout)
     /* Protect against late call from intf.c */
     aout->volume_set = NULL;
     aout->mute_set = NULL;
+    aout->device_select = NULL;
     aout_OutputUnlock (aout);
 
+    var_DelCallback (aout, "audio-filter", FilterCallback, NULL);
     var_DelCallback (aout, "mute", var_Copy, aout->p_parent);
     var_SetFloat (aout, "volume", -1.f);
     var_DelCallback (aout, "volume", var_Copy, aout->p_parent);
@@ -674,7 +684,7 @@ int aout_DeviceSet (audio_output_t *aout, const char *id)
  * The function will heap-allocate two tables of heap-allocated strings;
  * the caller is responsible for freeing all strings and both tables.
  *
- * \param ids pointer to a table of device identifiers [OUT]
+ * \param ids pointer to a table of device identifiers [OUT]
  * \param names pointer to a table of device human-readable descriptions [OUT]
  * \return the number of devices, or negative on error.
  * \note In case of error, *ids and *names are undefined.
