@@ -2,7 +2,7 @@
  * control.c
  *****************************************************************************
  * Copyright (C) 1999-2004 VLC authors and VideoLAN
- * $Id: d50ea973eed0c897c035ac14f9df5dc3ab1c4f88 $
+ * $Id: 2e4633ff689f575e6ef90e7e29274f79b1e4a6a4 $
  *
  * Authors: Gildas Bazin <gbazin@videolan.org>
  *
@@ -136,6 +136,15 @@ int input_vaControl( input_thread_t *p_input, int i_query, va_list args )
             i_64 = (int64_t)va_arg( args, int64_t );
             return var_SetTime( p_input, "spu-delay", i_64 );
 
+        case INPUT_NAV_ACTIVATE:
+        case INPUT_NAV_UP:
+        case INPUT_NAV_DOWN:
+        case INPUT_NAV_LEFT:
+        case INPUT_NAV_RIGHT:
+            input_ControlPush( p_input, i_query - INPUT_NAV_ACTIVATE
+                               + INPUT_CONTROL_NAV_ACTIVATE, NULL );
+            return VLC_SUCCESS;
+
         case INPUT_ADD_INFO:
         {
             char *psz_cat = (char *)va_arg( args, char * );
@@ -394,7 +403,7 @@ int input_vaControl( input_thread_t *p_input, int i_query, va_list args )
                 return VLC_EGENERIC;
             }
             *pi_attachment = p_input->p->i_attachment;
-            *ppp_attachment = malloc( sizeof(input_attachment_t**) * p_input->p->i_attachment );
+            *ppp_attachment = malloc( sizeof(input_attachment_t*) * p_input->p->i_attachment );
             for( i = 0; i < p_input->p->i_attachment; i++ )
                 (*ppp_attachment)[i] = vlc_input_attachment_Duplicate( p_input->p->attachment[i] );
 
@@ -521,28 +530,34 @@ static void UpdateBookmarksOption( input_thread_t *p_input )
     }
 
     char *psz_value = malloc( i_len + p_input->p->i_bookmark + 1 );
-    char *psz_next = psz_value;
 
-    psz_next += sprintf( psz_next, "bookmarks=" );
-    for( int i = 0; i < p_input->p->i_bookmark && psz_value != NULL; i++ )
+    if( psz_value != NULL )
     {
-        const seekpoint_t *p_bookmark = p_input->p->pp_bookmark[i];
+        strcpy( psz_value, "bookmarks=" );
 
-        psz_next += sprintf( psz_next, psz_format,
-                             p_bookmark->psz_name,
-                             p_bookmark->i_byte_offset,
-                             p_bookmark->i_time_offset/1000000 );
+        char *psz_next = psz_value + strlen( "bookmarks=" );
 
-        if( i < p_input->p->i_bookmark - 1)
-            *psz_next++ = ',';
+        for( int i = 0; i < p_input->p->i_bookmark && psz_value != NULL; i++ )
+        {
+            const seekpoint_t *p_bookmark = p_input->p->pp_bookmark[i];
+
+            psz_next += sprintf( psz_next, psz_format,
+                                 p_bookmark->psz_name,
+                                 p_bookmark->i_byte_offset,
+                                 p_bookmark->i_time_offset/1000000 );
+
+            if( i < p_input->p->i_bookmark - 1)
+                *psz_next++ = ',';
+        }
     }
     vlc_mutex_unlock( &p_input->p->p_item->lock );
 
-    if( psz_value )
-        input_item_AddOption( p_input->p->p_item, psz_value, VLC_INPUT_OPTION_UNIQUE );
-
-    free( psz_value );
-
+    if( psz_value != NULL )
+    {
+        input_item_AddOption( p_input->p->p_item, psz_value,
+                              VLC_INPUT_OPTION_UNIQUE );
+        free( psz_value );
+    }
     input_SendEventBookmark( p_input );
 }
 
