@@ -2,7 +2,7 @@
  * stream.c
  *****************************************************************************
  * Copyright (C) 1999-2004 VLC authors and VideoLAN
- * $Id: d3c4d9ec1132627879cb75c74ee9036528d9b9f9 $
+ * $Id: 4cceaac01b72d528516924a7e5342f09909bb692 $
  *
  * Authors: Laurent Aimar <fenrir@via.ecp.fr>
  *
@@ -318,7 +318,7 @@ stream_t *stream_AccessNew( access_t *p_access, char **ppsz_list )
         if( !p_entry )
             goto error;
 
-        p_entry->i_size = p_access->info.i_size;
+        p_entry->i_size = access_GetSize( p_access );
         p_entry->psz_path = strdup( p_access->psz_location );
         if( !p_entry->psz_path )
         {
@@ -328,7 +328,7 @@ stream_t *stream_AccessNew( access_t *p_access, char **ppsz_list )
         p_sys->p_list_access = p_access;
         TAB_APPEND( p_sys->i_list, p_sys->list, p_entry );
         msg_Dbg( p_access, "adding file `%s', (%"PRId64" bytes)",
-                 p_entry->psz_path, p_access->info.i_size );
+                 p_entry->psz_path, p_entry->i_size );
 
         for( int i = 0; ppsz_list[i] != NULL; i++ )
         {
@@ -340,17 +340,19 @@ stream_t *stream_AccessNew( access_t *p_access, char **ppsz_list )
             access_t *p_tmp = access_New( p_access, p_access->p_input,
                                           p_access->psz_access, "", psz_name );
             if( !p_tmp )
+            {
+                free( psz_name );
                 continue;
-
-            msg_Dbg( p_access, "adding file `%s', (%"PRId64" bytes)",
-                     psz_name, p_tmp->info.i_size );
+            }
 
             p_entry = malloc( sizeof(*p_entry) );
             if( p_entry )
             {
-                p_entry->i_size = p_tmp->info.i_size;
+                p_entry->i_size = access_GetSize( p_tmp );
                 p_entry->psz_path = psz_name;
                 TAB_APPEND( p_sys->i_list, p_sys->list, p_entry );
+                msg_Dbg( p_access, "adding file `%s', (%"PRId64" bytes)",
+                         p_entry->psz_path, p_entry->i_size );
             }
             access_Delete( p_tmp );
         }
@@ -540,6 +542,9 @@ static void AStreamControlUpdate( stream_t *s )
     }
 }
 
+#define static_control_match(foo) \
+    static_assert((unsigned) STREAM_##foo == ACCESS_##foo, "Mismatch")
+
 /****************************************************************************
  * AStreamControl:
  ****************************************************************************/
@@ -548,12 +553,46 @@ static int AStreamControl( stream_t *s, int i_query, va_list args )
     stream_sys_t *p_sys = s->p_sys;
     access_t     *p_access = p_sys->p_access;
 
-    uint64_t *pi_64, i_64;
+    static_control_match(CAN_SEEK);
+    static_control_match(CAN_FASTSEEK);
+    static_control_match(CAN_PAUSE);
+    static_control_match(CAN_CONTROL_PACE);
+    static_control_match(GET_PTS_DELAY);
+    static_control_match(GET_TITLE_INFO);
+    static_control_match(GET_TITLE);
+    static_control_match(GET_SEEKPOINT);
+    static_control_match(GET_META);
+    static_control_match(GET_CONTENT_TYPE);
+    static_control_match(GET_SIGNAL);
+    static_control_match(SET_PAUSE_STATE);
+    static_control_match(SET_TITLE);
+    static_control_match(SET_SEEKPOINT);
+    static_control_match(SET_PRIVATE_ID_STATE);
+    static_control_match(SET_PRIVATE_ID_CA);
+    static_control_match(GET_PRIVATE_ID_STATE);
 
     switch( i_query )
     {
+        case STREAM_CAN_SEEK:
+        case STREAM_CAN_FASTSEEK:
+        case STREAM_CAN_PAUSE:
+        case STREAM_CAN_CONTROL_PACE:
+        case STREAM_GET_PTS_DELAY:
+        case STREAM_GET_TITLE_INFO:
+        case STREAM_GET_TITLE:
+        case STREAM_GET_SEEKPOINT:
+        case STREAM_GET_META:
+        case STREAM_GET_CONTENT_TYPE:
+        case STREAM_GET_SIGNAL:
+        case STREAM_SET_PAUSE_STATE:
+        case STREAM_SET_PRIVATE_ID_STATE:
+        case STREAM_SET_PRIVATE_ID_CA:
+        case STREAM_GET_PRIVATE_ID_STATE:
+            return access_vaControl( p_access, i_query, args );
+
         case STREAM_GET_SIZE:
-            pi_64 = va_arg( args, uint64_t * );
+        {
+            uint64_t *pi_64 = va_arg( args, uint64_t * );
             if( s->p_sys->i_list )
             {
                 int i;
@@ -562,75 +601,37 @@ static int AStreamControl( stream_t *s, int i_query, va_list args )
                     *pi_64 += s->p_sys->list[i]->i_size;
                 break;
             }
-            *pi_64 = p_access->info.i_size;
+            *pi_64 = access_GetSize( p_access );
             break;
-
-        case STREAM_CAN_SEEK:
-            return access_vaControl( p_access, ACCESS_CAN_SEEK, args );
-        case STREAM_CAN_FASTSEEK:
-            return access_vaControl( p_access, ACCESS_CAN_FASTSEEK, args );
-        case STREAM_CAN_PAUSE:
-            return access_vaControl( p_access, ACCESS_CAN_PAUSE, args );
-        case STREAM_CAN_CONTROL_PACE:
-            return access_vaControl( p_access, ACCESS_CAN_CONTROL_PACE, args );
+        }
 
         case STREAM_GET_POSITION:
-            pi_64 = va_arg( args, uint64_t * );
-            *pi_64 = p_sys->i_pos;
+            *va_arg( args, uint64_t * ) = p_sys->i_pos;
             break;
 
         case STREAM_SET_POSITION:
-            i_64 = va_arg( args, uint64_t );
+        {
+            uint64_t offset = va_arg( args, uint64_t );
             switch( p_sys->method )
             {
             case STREAM_METHOD_BLOCK:
-                return AStreamSeekBlock( s, i_64 );
+                return AStreamSeekBlock( s, offset );
             case STREAM_METHOD_STREAM:
-                return AStreamSeekStream( s, i_64 );
+                return AStreamSeekStream( s, offset );
             default:
                 assert(0);
                 return VLC_EGENERIC;
             }
-
-        case STREAM_CONTROL_ACCESS:
-        {
-            int i_int = (int) va_arg( args, int );
-            if( i_int != ACCESS_SET_PRIVATE_ID_STATE &&
-                i_int != ACCESS_SET_PRIVATE_ID_CA &&
-                i_int != ACCESS_GET_PRIVATE_ID_STATE )
-            {
-                msg_Err( s, "Hey, what are you thinking ?"
-                            "DON'T USE STREAM_CONTROL_ACCESS !!!" );
-                return VLC_EGENERIC;
-            }
-            return access_vaControl( p_access, i_int, args );
         }
 
         case STREAM_UPDATE_SIZE:
             AStreamControlUpdate( s );
             return VLC_SUCCESS;
 
-        case STREAM_GET_TITLE_INFO:
-            return access_vaControl( p_access, ACCESS_GET_TITLE_INFO, args );
-        case STREAM_GET_META:
-            return access_vaControl( p_access, ACCESS_GET_META, args );
-        case STREAM_GET_CONTENT_TYPE:
-            return access_vaControl( p_access, ACCESS_GET_CONTENT_TYPE, args );
-        case STREAM_GET_SIGNAL:
-            return access_vaControl( p_access, ACCESS_GET_SIGNAL, args );
-
-        case STREAM_SET_PAUSE_STATE:
-            return access_vaControl( p_access, ACCESS_SET_PAUSE_STATE, args );
         case STREAM_SET_TITLE:
-        {
-            int ret = access_vaControl( p_access, ACCESS_SET_TITLE, args );
-            if( ret == VLC_SUCCESS )
-                AStreamControlReset( s );
-            return ret;
-        }
         case STREAM_SET_SEEKPOINT:
         {
-            int ret = access_vaControl( p_access, ACCESS_SET_SEEKPOINT, args );
+            int ret = access_vaControl( p_access, i_query, args );
             if( ret == VLC_SUCCESS )
                 AStreamControlReset( s );
             return ret;
@@ -1082,15 +1083,15 @@ static int AStreamPeekStream( stream_t *s, const uint8_t **pp_peek, unsigned int
             /* Be sure we will read something */
             p_sys->stream.i_used += tk->i_start + p_sys->stream.i_offset + i_read - tk->i_end;
         }
-        if( AStreamRefillStream( s ) ) break;
+        if( AStreamRefillStream( s ) )
+        {
+            if( tk->i_end < tk->i_start + p_sys->stream.i_offset )
+                return 0; /* EOF */
+            i_read = tk->i_end - tk->i_start - p_sys->stream.i_offset;
+            break;
+        }
     }
 
-    if( tk->i_end < tk->i_start + p_sys->stream.i_offset + i_read )
-    {
-        i_read = tk->i_end - tk->i_start - p_sys->stream.i_offset;
-    }
-
-
     /* Now, direct pointer or a copy ? */
     i_off = (tk->i_start + p_sys->stream.i_offset) % STREAM_CACHE_TRACK_SIZE;
     if( i_off + i_read <= STREAM_CACHE_TRACK_SIZE )
