@@ -2,7 +2,7 @@
  * image.c : wrapper for image reading/writing facilities
  *****************************************************************************
  * Copyright (C) 2004-2007 VLC authors and VideoLAN
- * $Id: 2ce3f9ca691c42c6de34f7fcce52cd37fdd0b1a1 $
+ * $Id: 0f25609ee215df5d22850b6f6996c9c9d896ed3c $
  *
  * Author: Gildas Bazin <gbazin@videolan.org>
  *
@@ -174,6 +174,10 @@ static picture_t *ImageRead( image_handler_t *p_image, block_t *p_block,
         p_fmt_out->i_width = p_image->p_dec->fmt_out.video.i_width;
     if( !p_fmt_out->i_height )
         p_fmt_out->i_height = p_image->p_dec->fmt_out.video.i_height;
+    if( !p_fmt_out->i_visible_width )
+        p_fmt_out->i_visible_width = p_fmt_out->i_width;
+    if( !p_fmt_out->i_visible_height )
+        p_fmt_out->i_visible_height = p_fmt_out->i_height;
 
     /* Check if we need chroma conversion or resizing */
     if( p_image->p_dec->fmt_out.video.i_chroma != p_fmt_out->i_chroma ||
@@ -381,7 +385,7 @@ static int ImageWriteUrl( image_handler_t *p_image, picture_t *p_pic,
     file = vlc_fopen( psz_url, "wb" );
     if( !file )
     {
-        msg_Err( p_image->p_parent, "%s: %m", psz_url );
+        msg_Err( p_image->p_parent, "%s: %s", psz_url, vlc_strerror_c(errno) );
         return VLC_EGENERIC;
     }
 
@@ -401,7 +405,7 @@ static int ImageWriteUrl( image_handler_t *p_image, picture_t *p_pic,
     if( err )
     {
        errno = err;
-       msg_Err( p_image->p_parent, "%s: %m", psz_url );
+       msg_Err( p_image->p_parent, "%s: %s", psz_url, vlc_strerror_c(errno) );
     }
 
     return err ? VLC_EGENERIC : VLC_SUCCESS;
@@ -550,6 +554,7 @@ static const struct
     { VLC_FOURCC('x','c','f',' '), "xcf" },
     { VLC_CODEC_PCX,               "pcx" },
     { VLC_CODEC_GIF,               "gif" },
+    { VLC_CODEC_SVG,               "svg" },
     { VLC_CODEC_TIFF,              "tif" },
     { VLC_CODEC_TIFF,              "tiff" },
     { VLC_FOURCC('l','b','m',' '), "lbm" },
@@ -603,6 +608,7 @@ static const struct
     { VLC_CODEC_JPEG,              "image/jpeg" },
     { VLC_CODEC_PCX,               "image/pcx" },
     { VLC_CODEC_PNG,               "image/png" },
+    { VLC_CODEC_SVG,               "image/svg+xml" },
     { VLC_CODEC_TIFF,              "image/tiff" },
     { VLC_CODEC_TARGA,             "image/x-tga" },
     { VLC_FOURCC('x','p','m',' '), "image/x-xpixmap" },
@@ -703,26 +709,32 @@ static encoder_t *CreateEncoder( vlc_object_t *p_this, video_format_t *fmt_in,
     p_enc->p_module = NULL;
     es_format_Init( &p_enc->fmt_in, VIDEO_ES, fmt_in->i_chroma );
     p_enc->fmt_in.video = *fmt_in;
-    if( fmt_out->i_width > 0 && fmt_out->i_height > 0 )
-    {
-        p_enc->fmt_in.video.i_width = fmt_out->i_width;
-        p_enc->fmt_in.video.i_height = fmt_out->i_height;
 
-        if( fmt_out->i_visible_width > 0 &&
-            fmt_out->i_visible_height > 0 )
+    if( p_enc->fmt_in.video.i_visible_width == 0 ||
+        p_enc->fmt_in.video.i_visible_height == 0 ||
+        p_enc->fmt_out.video.i_visible_width == 0 ||
+        p_enc->fmt_out.video.i_visible_height == 0 )
+    {
+        if( fmt_out->i_width > 0 && fmt_out->i_height > 0 )
         {
-            p_enc->fmt_in.video.i_visible_width = fmt_out->i_visible_width;
-            p_enc->fmt_in.video.i_visible_height = fmt_out->i_visible_height;
+            p_enc->fmt_in.video.i_width = fmt_out->i_width;
+            p_enc->fmt_in.video.i_height = fmt_out->i_height;
+
+            if( fmt_out->i_visible_width > 0 &&
+                fmt_out->i_visible_height > 0 )
+            {
+                p_enc->fmt_in.video.i_visible_width = fmt_out->i_visible_width;
+                p_enc->fmt_in.video.i_visible_height = fmt_out->i_visible_height;
+            }
+            else
+            {
+                p_enc->fmt_in.video.i_visible_width = fmt_out->i_width;
+                p_enc->fmt_in.video.i_visible_height = fmt_out->i_height;
+            }
         }
-        else
-        {
-            p_enc->fmt_in.video.i_visible_width = fmt_out->i_width;
-            p_enc->fmt_in.video.i_visible_height = fmt_out->i_height;
-        }
-    }
-    else if( fmt_out->i_sar_num && fmt_out->i_sar_den &&
-             fmt_out->i_sar_num * fmt_in->i_sar_den !=
-             fmt_out->i_sar_den * fmt_in->i_sar_num )
+    } else if( fmt_out->i_sar_num && fmt_out->i_sar_den &&
+               fmt_out->i_sar_num * fmt_in->i_sar_den !=
+               fmt_out->i_sar_den * fmt_in->i_sar_num )
     {
         p_enc->fmt_in.video.i_width =
             fmt_in->i_sar_num * (int64_t)fmt_out->i_sar_den * fmt_in->i_width /
