@@ -35,9 +35,7 @@
 
 #include <sys/types.h>
 #include <sys/stat.h>
-#ifdef HAVE_UNISTD_H
-#   include <unistd.h>
-#endif
+#include <unistd.h>
 
 #include <vlc_common.h>
 #include <vlc_plugin.h>
@@ -104,12 +102,12 @@ void module_InitBank (void)
 
     if (modules.usage == 0)
     {
-        /* Fills the module bank structure with the main module infos.
-         * This is very useful as it will allow us to consider the main
+        /* Fills the module bank structure with the core module infos.
+         * This is very useful as it will allow us to consider the core
          * library just as another module, and for instance the configuration
-         * options of main will be available in the module bank structure just
+         * options of core will be available in the module bank structure just
          * as for every other module. */
-        module_t *module = module_InitStatic (vlc_entry__main);
+        module_t *module = module_InitStatic (vlc_entry__core);
         if (likely(module != NULL))
             module_StoreBank (module);
         config_SortConfig ();
@@ -119,7 +117,7 @@ void module_InitBank (void)
     /* We do retain the module bank lock until the plugins are loaded as well.
      * This is ugly, this staged loading approach is needed: LibVLC gets
      * some configuration parameters relevant to loading the plugins from
-     * the main (builtin) module. The module bank becomes shared read-only data
+     * the core (builtin) module. The module bank becomes shared read-only data
      * once it is ready, so we need to fully serialize initialization.
      * DO NOT UNCOMMENT the following line unless you managed to squeeze
      * module_LoadPlugins() before you unlock the mutex. */
@@ -413,6 +411,9 @@ static void AllocatePluginPath (vlc_object_t *p_this, const char *path,
                 free (cache[i].path);
             }
             free( cache );
+#ifdef __APPLE__
+            break;
+#endif
         case CACHE_RESET:
             CacheSave (p_this, path, bank.cache, bank.i_cache);
         case CACHE_IGNORE:
@@ -446,7 +447,7 @@ static void AllocatePluginDir (module_bank_t *bank, unsigned maxdepth,
 
         /* Skip ".", ".." */
         if (!strcmp (file, ".") || !strcmp (file, ".."))
-            goto skip;
+            continue;
 
         /* Compute path relative to plug-in base directory */
         if (reldir != NULL)
@@ -457,7 +458,7 @@ static void AllocatePluginDir (module_bank_t *bank, unsigned maxdepth,
         else
             relpath = strdup (file);
         if (unlikely(relpath == NULL))
-            goto skip;
+            continue;
 
         /* Compute absolute path */
         if (asprintf (&abspath, "%s"DIR_SEP"%s", bank->base, relpath) == -1)
@@ -495,7 +496,6 @@ static void AllocatePluginDir (module_bank_t *bank, unsigned maxdepth,
     skip:
         free (relpath);
         free (abspath);
-        free (file);
     }
     closedir (dh);
 }
