@@ -150,19 +150,6 @@ gai_error_from_herrno (void)
     return EAI_SYSTEM;
 }
 
-/*
- * This functions must be used to free the memory allocated by getaddrinfo().
- */
-void freeaddrinfo (struct addrinfo *res)
-{
-    if (res == NULL)
-        return;
-    free (res->ai_canonname);
-    free (res->ai_addr);
-    free (res->ai_next);
-    free (res);
-}
-
 /*
  * Internal function that builds an addrinfo struct.
  */
@@ -261,7 +248,7 @@ getaddrinfo (const char *node, const char *service,
                 protocol = IPPROTO_UDP;
                 break;
 
-#ifndef SOCK_RAW
+#ifdef SOCK_RAW
             case SOCK_RAW:
 #endif
             case 0:
