@@ -45,6 +45,8 @@
 #include <sys/time.h>
 #include <sys/select.h>
 
+#include <sys/builtin.h>
+
 static vlc_threadvar_t thread_key;
 
 /**
@@ -68,7 +70,7 @@ struct vlc_thread
 
 static void vlc_cancel_self (PVOID dummy);
 
-static ULONG vlc_DosWaitEventSemEx( HEV hev, ULONG ulTimeout, BOOL fCancelable )
+static ULONG vlc_DosWaitEventSemEx( HEV hev, ULONG ulTimeout )
 {
     HMUX      hmux;
     SEMRECORD asr[ 2 ];
@@ -77,10 +79,11 @@ static ULONG vlc_DosWaitEventSemEx( HEV hev, ULONG ulTimeout, BOOL fCancelable )
     ULONG     rc;
 
     struct vlc_thread *th = vlc_threadvar_get( thread_key );
-    if( th == NULL || !fCancelable )
+    if( th == NULL || !th->killable )
     {
         /* Main thread - cannot be cancelled anyway
          * Alien thread - out of our control
+         * Cancel disabled thread - ignore cancel
          */
         if( hev != NULLHANDLE )
             return DosWaitEventSem( hev, ulTimeout );
@@ -116,12 +119,12 @@ static ULONG vlc_DosWaitEventSemEx( HEV hev, ULONG ulTimeout, BOOL fCancelable )
 
 static ULONG vlc_WaitForSingleObject (HEV hev, ULONG ulTimeout)
 {
-    return vlc_DosWaitEventSemEx( hev, ulTimeout, TRUE );
+    return vlc_DosWaitEventSemEx( hev, ulTimeout );
 }
 
 static ULONG vlc_Sleep (ULONG ulTimeout)
 {
-    ULONG rc = vlc_DosWaitEventSemEx( NULLHANDLE, ulTimeout, TRUE );
+    ULONG rc = vlc_DosWaitEventSemEx( NULLHANDLE, ulTimeout );
 
     return ( rc != ERROR_TIMEOUT ) ? rc : 0;
 }
@@ -261,9 +264,12 @@ enum
 
 static void vlc_cond_init_common (vlc_cond_t *p_condvar, unsigned clock)
 {
-    /* Create a manual-reset event (manual reset is needed for broadcast). */
-    if (DosCreateEventSem (NULL, &p_condvar->hev, 0, FALSE))
+    if (DosCreateEventSem (NULL, &p_condvar->hev, 0, FALSE) ||
+        DosCreateEventSem (NULL, &p_condvar->hevAck, 0, FALSE))
         abort();
+
+    p_condvar->waiters = 0;
+    p_condvar->signaled = 0;
     p_condvar->clock = clock;
 }
 
@@ -280,6 +286,7 @@ void vlc_cond_init_daytime (vlc_cond_t *p_condvar)
 void vlc_cond_destroy (vlc_cond_t *p_condvar)
 {
     DosCloseEventSem( p_condvar->hev );
+    DosCloseEventSem( p_condvar->hevAck );
 }
 
 void vlc_cond_signal (vlc_cond_t *p_condvar)
@@ -287,8 +294,16 @@ void vlc_cond_signal (vlc_cond_t *p_condvar)
     if (!p_condvar->hev)
         return;
 
-    /* This is suboptimal but works. */
-    vlc_cond_broadcast (p_condvar);
+    if (!__atomic_cmpxchg32 (&p_condvar->waiters, 0, 0))
+    {
+        ULONG ulPost;
+
+        __atomic_xchg (&p_condvar->signaled, 1);
+        DosPostEventSem (p_condvar->hev);
+
+        DosWaitEventSem (p_condvar->hevAck, SEM_INDEFINITE_WAIT);
+        DosResetEventSem (p_condvar->hevAck, &ulPost);
+    }
 }
 
 void vlc_cond_broadcast (vlc_cond_t *p_condvar)
@@ -296,39 +311,57 @@ void vlc_cond_broadcast (vlc_cond_t *p_condvar)
     if (!p_condvar->hev)
         return;
 
-    /* Wake all threads up (as the event HANDLE has manual reset) */
-    DosPostEventSem( p_condvar->hev );
+    while (!__atomic_cmpxchg32 (&p_condvar->waiters, 0, 0))
+        vlc_cond_signal (p_condvar);
 }
 
-void vlc_cond_wait (vlc_cond_t *p_condvar, vlc_mutex_t *p_mutex)
+static int vlc_cond_wait_common (vlc_cond_t *p_condvar, vlc_mutex_t *p_mutex,
+                                 ULONG ulTimeout)
 {
     ULONG ulPost;
     ULONG rc;
 
+    do
+    {
+        vlc_testcancel();
+
+        __atomic_increment (&p_condvar->waiters);
+
+        vlc_mutex_unlock (p_mutex);
+
+        do
+        {
+            rc = vlc_WaitForSingleObject( p_condvar->hev, ulTimeout );
+            if (rc == NO_ERROR)
+                DosResetEventSem (p_condvar->hev, &ulPost);
+        } while (rc == NO_ERROR &&
+                 __atomic_cmpxchg32 (&p_condvar->signaled, 0, 1) == 0);
+
+        __atomic_decrement (&p_condvar->waiters);
+
+        DosPostEventSem (p_condvar->hevAck);
+
+        vlc_mutex_lock (p_mutex);
+    } while( rc == ERROR_INTERRUPT );
+
+    return rc ? ETIMEDOUT : 0;
+}
+
+void vlc_cond_wait (vlc_cond_t *p_condvar, vlc_mutex_t *p_mutex)
+{
     if (!p_condvar->hev)
     {   /* FIXME FIXME FIXME */
         msleep (50000);
         return;
     }
 
-    do
-    {
-        vlc_testcancel();
-
-        vlc_mutex_unlock (p_mutex);
-        rc = vlc_WaitForSingleObject( p_condvar->hev, SEM_INDEFINITE_WAIT );
-        vlc_mutex_lock (p_mutex);
-    } while( rc == ERROR_INTERRUPT );
-
-    DosResetEventSem( p_condvar->hev, &ulPost );
+    vlc_cond_wait_common (p_condvar, p_mutex, SEM_INDEFINITE_WAIT);
 }
 
 int vlc_cond_timedwait (vlc_cond_t *p_condvar, vlc_mutex_t *p_mutex,
                         mtime_t deadline)
 {
     ULONG   ulTimeout;
-    ULONG   ulPost;
-    ULONG   rc;
 
     if (!p_condvar->hev)
     {   /* FIXME FIXME FIXME */
@@ -336,35 +369,30 @@ int vlc_cond_timedwait (vlc_cond_t *p_condvar, vlc_mutex_t *p_mutex,
         return 0;
     }
 
-    do
+    mtime_t total;
+    switch (p_condvar->clock)
     {
-        vlc_testcancel();
-
-        mtime_t total;
-        switch (p_condvar->clock)
+        case CLOCK_REALTIME:
         {
-            case CLOCK_REALTIME: /* FIXME? sub-second precision */
-                total = CLOCK_FREQ * time (NULL);
-                break;
-            default:
-                assert (p_condvar->clock == CLOCK_MONOTONIC);
-                total = mdate();
-                break;
+            struct timeval tv;
+            gettimeofday (&tv, NULL);
+
+            total = CLOCK_FREQ * tv.tv_sec +
+                    CLOCK_FREQ * tv.tv_usec / 1000000L;
+            break;
         }
-        total = (deadline - total) / 1000;
-        if( total < 0 )
-            total = 0;
+        default:
+            assert (p_condvar->clock == CLOCK_MONOTONIC);
+            total = mdate();
+            break;
+    }
+    total = (deadline - total) / 1000;
+    if( total < 0 )
+        total = 0;
 
-        ulTimeout = ( total > 0x7fffffff ) ? 0x7fffffff : total;
+    ulTimeout = ( total > 0x7fffffff ) ? 0x7fffffff : total;
 
-        vlc_mutex_unlock (p_mutex);
-        rc = vlc_WaitForSingleObject( p_condvar->hev, ulTimeout );
-        vlc_mutex_lock (p_mutex);
-    } while( rc == ERROR_INTERRUPT );
-
-    DosResetEventSem( p_condvar->hev, &ulPost );
-
-    return rc ? ETIMEDOUT : 0;
+    return vlc_cond_wait_common (p_condvar, p_mutex, ulTimeout);
 }
 
 /*** Thread-specific variables (TLS) ***/
