@@ -1,8 +1,8 @@
 /*****************************************************************************
- * preparse.c: Preparser thread.
+ * preparser.c: Preparser thread.
  *****************************************************************************
  * Copyright © 1999-2009 VLC authors and VideoLAN
- * $Id: ba9a8d979c7046d8ed7f78396c02d2d5c5c51b87 $
+ * $Id: b44b9f8db8efcdac25f0bf51cf610f32d8ed92a8 $
  *
  * Authors: Samuel Hocevar <sam@zoy.org>
  *          Clément Stenac <zorglub@videolan.org>
@@ -26,20 +26,17 @@
 #endif
 
 #include <vlc_common.h>
-#include <vlc_playlist.h>
 
-#include "art.h"
 #include "fetcher.h"
 #include "preparser.h"
-#include "../input/input_interface.h"
-
+#include "input/input_interface.h"
 
 /*****************************************************************************
  * Structures/definitions
  *****************************************************************************/
 struct playlist_preparser_t
 {
-    playlist_t          *p_playlist;
+    vlc_object_t        *object;
     playlist_fetcher_t  *p_fetcher;
 
     vlc_mutex_t     lock;
@@ -47,8 +44,6 @@ struct playlist_preparser_t
     bool            b_live;
     input_item_t  **pp_waiting;
     int             i_waiting;
-
-    int             i_art_policy;
 };
 
 static void *Thread( void * );
@@ -56,27 +51,31 @@ static void *Thread( void * );
 /*****************************************************************************
  * Public functions
  *****************************************************************************/
-playlist_preparser_t *playlist_preparser_New( playlist_t *p_playlist, playlist_fetcher_t *p_fetcher )
+playlist_preparser_t *playlist_preparser_New( vlc_object_t *parent )
 {
     playlist_preparser_t *p_preparser = malloc( sizeof(*p_preparser) );
     if( !p_preparser )
         return NULL;
 
-    p_preparser->p_playlist = p_playlist;
-    p_preparser->p_fetcher = p_fetcher;
+    p_preparser->object = parent;
+    p_preparser->p_fetcher = playlist_fetcher_New( parent );
+    if( unlikely(p_preparser->p_fetcher == NULL) )
+        msg_Err( parent, "cannot create fetcher" );
+
     vlc_mutex_init( &p_preparser->lock );
     vlc_cond_init( &p_preparser->wait );
     p_preparser->b_live = false;
-    p_preparser->i_art_policy = var_GetInteger( p_playlist, "album-art" );
     p_preparser->i_waiting = 0;
     p_preparser->pp_waiting = NULL;
 
     return p_preparser;
 }
 
-void playlist_preparser_Push( playlist_preparser_t *p_preparser, input_item_t *p_item )
+void playlist_preparser_Push( playlist_preparser_t *p_preparser, input_item_t *p_item,
+                              input_item_meta_request_option_t i_options )
 {
     vlc_gc_incref( p_item );
+    VLC_UNUSED( i_options );
 
     vlc_mutex_lock( &p_preparser->lock );
     INSERT_ELEM( p_preparser->pp_waiting, p_preparser->i_waiting,
@@ -85,14 +84,20 @@ void playlist_preparser_Push( playlist_preparser_t *p_preparser, input_item_t *p
     {
         if( vlc_clone_detach( NULL, Thread, p_preparser,
                               VLC_THREAD_PRIORITY_LOW ) )
-            msg_Warn( p_preparser->p_playlist,
-                      "cannot spawn pre-parser thread" );
+            msg_Warn( p_preparser->object, "cannot spawn pre-parser thread" );
         else
             p_preparser->b_live = true;
     }
     vlc_mutex_unlock( &p_preparser->lock );
 }
 
+void playlist_preparser_fetcher_Push( playlist_preparser_t *p_preparser,
+             input_item_t *p_item, input_item_meta_request_option_t i_options )
+{
+    if( p_preparser->p_fetcher != NULL )
+        playlist_fetcher_Push( p_preparser->p_fetcher, p_item, i_options );
+}
+
 void playlist_preparser_Delete( playlist_preparser_t *p_preparser )
 {
     vlc_mutex_lock( &p_preparser->lock );
@@ -110,6 +115,9 @@ void playlist_preparser_Delete( playlist_preparser_t *p_preparser )
     /* Destroy the item preparser */
     vlc_cond_destroy( &p_preparser->wait );
     vlc_mutex_destroy( &p_preparser->lock );
+
+    if( p_preparser->p_fetcher != NULL )
+        playlist_fetcher_Delete( p_preparser->p_fetcher );
     free( p_preparser );
 }
 
@@ -119,7 +127,7 @@ void playlist_preparser_Delete( playlist_preparser_t *p_preparser )
 /**
  * This function preparses an item when needed.
  */
-static void Preparse( playlist_t *p_playlist, input_item_t *p_item )
+static void Preparse( vlc_object_t *obj, input_item_t *p_item )
 {
     vlc_mutex_lock( &p_item->lock );
     int i_type = p_item->i_type;
@@ -131,18 +139,14 @@ static void Preparse( playlist_t *p_playlist, input_item_t *p_item )
         return;
     }
 
-    stats_TimerStart( p_playlist, "Preparse run", STATS_TIMER_PREPARSE );
-
     /* Do not preparse if it is already done (like by playing it) */
     if( !input_item_IsPreparsed( p_item ) )
     {
-        input_Preparse( VLC_OBJECT(p_playlist), p_item );
+        input_Preparse( obj, p_item );
         input_item_SetPreparsed( p_item, true );
 
-        var_SetAddress( p_playlist, "item-change", p_item );
+        var_SetAddress( obj, "item-change", p_item );
     }
-
-    stats_TimerStop( p_playlist, STATS_TIMER_PREPARSE );
 }
 
 /**
@@ -150,7 +154,7 @@ static void Preparse( playlist_t *p_playlist, input_item_t *p_item )
  */
 static void Art( playlist_preparser_t *p_preparser, input_item_t *p_item )
 {
-    playlist_t *p_playlist = p_preparser->p_playlist;
+    vlc_object_t *obj = p_preparser->object;
     playlist_fetcher_t *p_fetcher = p_preparser->p_fetcher;
 
     bool b_fetch = false;
@@ -166,18 +170,16 @@ static void Art( playlist_preparser_t *p_preparser, input_item_t *p_item )
         const char *psz_arturl = vlc_meta_Get( p_item->p_meta, vlc_meta_ArtworkURL );
         const char *psz_name = vlc_meta_Get( p_item->p_meta, vlc_meta_Title );
 
-        if( p_preparser->i_art_policy == ALBUM_ART_ALL &&
-                ( !psz_arturl ||
-                  ( strncmp( psz_arturl, "file://", 7 ) &&
-                    strncmp( psz_arturl, "attachment://", 13 ) ) ) )
+        if( !psz_arturl || ( strncmp( psz_arturl, "file://", 7 ) &&
+                             strncmp( psz_arturl, "attachment://", 13 ) ) )
         {
-            msg_Dbg( p_playlist, "meta ok for %s, need to fetch art",
+            msg_Dbg( obj, "meta ok for %s, need to fetch art",
                      psz_name ? psz_name : "(null)" );
             b_fetch = true;
         }
         else
         {
-            msg_Dbg( p_playlist, "no fetch required for %s (art currently %s)",
+            msg_Dbg( obj, "no fetch required for %s (art currently %s)",
                      psz_name ? psz_name : "(null)",
                      psz_arturl ? psz_arturl : "(null)" );
         }
@@ -185,7 +187,7 @@ static void Art( playlist_preparser_t *p_preparser, input_item_t *p_item )
     vlc_mutex_unlock( &p_item->lock );
 
     if( b_fetch && p_fetcher )
-        playlist_fetcher_Push( p_fetcher, p_item );
+        playlist_fetcher_Push( p_fetcher, p_item, 0 );
 }
 
 /**
@@ -194,7 +196,7 @@ static void Art( playlist_preparser_t *p_preparser, input_item_t *p_item )
 static void *Thread( void *data )
 {
     playlist_preparser_t *p_preparser = data;
-    playlist_t *p_playlist = p_preparser->p_playlist;
+    vlc_object_t *obj = p_preparser->object;
 
     for( ;; )
     {
@@ -218,7 +220,7 @@ static void *Thread( void *data )
         if( !p_current )
             break;
 
-        Preparse( p_playlist, p_current );
+        Preparse( obj, p_current );
 
         Art( p_preparser, p_current );
         vlc_gc_decref(p_current);
