@@ -2,7 +2,7 @@
  * announce.c : announce handler
  *****************************************************************************
  * Copyright (C) 2002-2007 VLC authors and VideoLAN
- * $Id: e7f39e77243d894b548a4ed0a6a8024de5fdb13e $
+ * $Id: 7bebdf2476f531ba47a9724a2de025a3badc3417 $
  *
  * Authors: Clément Stenac <zorglub@videolan.org>
  *
@@ -68,7 +68,7 @@ sout_AnnounceRegisterSDP( vlc_object_t *obj, const char *psz_sdp,
 
     /* GRUIK. We should not convert back-and-forth from string to numbers */
     struct addrinfo *res;
-    if (vlc_getaddrinfo (obj, psz_dst, 0, NULL, &res) == 0)
+    if (vlc_getaddrinfo (psz_dst, 0, NULL, &res) == 0)
     {
         if (res->ai_addrlen <= sizeof (p_session->addr))
             memcpy (&p_session->addr, res->ai_addr,
