@@ -2,7 +2,7 @@
  * snapshot.c : vout internal snapshot
  *****************************************************************************
  * Copyright (C) 2009 Laurent Aimar
- * $Id: 5accd89329dcd69dcf8bc89dcc149b0c0cd31e17 $
+ * $Id: 7b89803740174cb81ad09718d4a7f997d37a6f18 $
  *
  * Authors: Gildas Bazin <gbazin _AT_ videolan _DOT_ org>
  *          Laurent Aimar <fenrir _AT_ videolan _DOT_ org>
@@ -26,8 +26,10 @@
 # include "config.h"
 #endif
 
+#include <assert.h>
 #include <sys/stat.h>
 #include <sys/types.h>
+#include <sys/time.h>
 #include <dirent.h>
 #include <time.h>
 
@@ -35,8 +37,10 @@
 #include <vlc_fs.h>
 #include <vlc_strings.h>
 #include <vlc_block.h>
+#include <vlc_vout.h>
 
 #include "snapshot.h"
+#include "vout_internal.h"
 
 /* */
 void vout_snapshot_Init(vout_snapshot_t *snap)
@@ -136,12 +140,13 @@ char *vout_snapshot_GetDirectory(void)
 /* */
 int vout_snapshot_SaveImage(char **name, int *sequential,
                              const block_t *image,
-                             vlc_object_t *object,
+                             vout_thread_t *p_vout,
                              const vout_snapshot_save_cfg_t *cfg)
 {
     /* */
     char *filename;
     DIR *pathdir = vlc_opendir(cfg->path);
+    input_thread_t *input = (input_thread_t*)p_vout->p->input;
     if (pathdir != NULL) {
         /* The use specified a directory path */
         closedir(pathdir);
@@ -149,7 +154,7 @@ int vout_snapshot_SaveImage(char **name, int *sequential,
         /* */
         char *prefix = NULL;
         if (cfg->prefix_fmt)
-            prefix = str_format_time(cfg->prefix_fmt);
+            prefix = str_format(input, cfg->prefix_fmt);
         if (prefix)
             filename_sanitize(prefix);
         else {
@@ -174,35 +179,26 @@ int vout_snapshot_SaveImage(char **name, int *sequential,
                 free(filename);
             }
         } else {
-            struct tm    curtime;
-            time_t       lcurtime = time(NULL) ;
+            struct timeval tv;
+            struct tm curtime;
+            char buffer[128];
 
-            if (!localtime_r(&lcurtime, &curtime)) {
-                const unsigned int id = (image->i_pts / 100000) & 0xFFFFFF;
+            gettimeofday(&tv, NULL);
+            if (localtime_r(&tv.tv_sec, &curtime) == NULL)
+                gmtime_r(&tv.tv_sec, &curtime);
+            if (strftime(buffer, sizeof(buffer), "%Y-%m-%d-%Hh%Mm%Ss",
+                         &curtime) == 0)
+                strcpy(buffer, "error");
 
-                msg_Warn(object, "failed to get current time. Falling back to legacy snapshot naming");
-
-                if (asprintf(&filename, "%s" DIR_SEP "%s%u.%s",
-                             cfg->path, prefix, id, cfg->format) < 0)
-                    filename = NULL;
-            } else {
-                /* suffix with the last decimal digit in 10s of seconds resolution
-                 * FIXME gni ? */
-                const int id = (image->i_pts / (100*1000)) & 0xFF;
-                char buffer[128];
-
-                if (!strftime(buffer, sizeof(buffer), "%Y-%m-%d-%Hh%Mm%Ss", &curtime))
-                    strcpy(buffer, "error");
-
-                if (asprintf(&filename, "%s" DIR_SEP "%s%s%1u.%s",
-                             cfg->path, prefix, buffer, id, cfg->format) < 0)
-                    filename = NULL;
-            }
+            if (asprintf(&filename, "%s" DIR_SEP "%s%s%03u.%s",
+                         cfg->path, prefix, buffer,
+                         (unsigned)tv.tv_usec / 1000, cfg->format) < 0)
+                filename = NULL;
         }
         free(prefix);
     } else {
         /* The user specified a full path name (including file name) */
-        filename = str_format_time(cfg->path);
+        filename = str_format(input, cfg->path);
         path_sanitize(filename);
     }
 
@@ -212,12 +208,12 @@ int vout_snapshot_SaveImage(char **name, int *sequential,
     /* Save the snapshot */
     FILE *file = vlc_fopen(filename, "wb");
     if (!file) {
-        msg_Err(object, "Failed to open '%s'", filename);
+        msg_Err(p_vout, "Failed to open '%s'", filename);
         free(filename);
         goto error;
     }
     if (fwrite(image->p_buffer, image->i_buffer, 1, file) != 1) {
-        msg_Err(object, "Failed to write to '%s'", filename);
+        msg_Err(p_vout, "Failed to write to '%s'", filename);
         fclose(file);
         free(filename);
         goto error;
@@ -233,7 +229,7 @@ int vout_snapshot_SaveImage(char **name, int *sequential,
     return VLC_SUCCESS;
 
 error:
-    msg_Err(object, "could not save snapshot");
+    msg_Err(p_vout, "could not save snapshot");
     return VLC_EGENERIC;
 }
 
